from openfisca_france.model.base import Variable, Individu, MONTH, TypesActivite, max_, set_input_dispatch_by_period, set_input_divide_by_period


class depart1825_eligibilite(Variable):
    '''
    Situations non prises en compte
    - en contrat aidé,
    - inscrit dans une école de la deuxième chance,
    - volontaire en service civique,
    - jeunes suivis par l'Aide Sociale à l'Enfance.​
    '''
    value_type = bool
    label = 'Éligibilité au dispositif départ 18-25'
    entity = Individu
    definition_period = MONTH
    set_input = set_input_dispatch_by_period
    reference = [
        'https://programme-depart-1825.com/eligibilite/',
        'https://www.ancv.com/actualites/le-magazine/depart-1825-un-nouveau-programme-pour-les-jeunes-de-18-25-ans'
        ]

    def formula(individu, period, parameters):
        criteres_age = parameters(period).prestations_sociales.aides_jeunes.depart1825.age
        age = individu('age', period)

        eligibilite_age = (criteres_age.minimum <= age) * (age <= criteres_age.maximum)

        etudiant_boursier = (individu('activite', period) == TypesActivite.etudiant) * individu('boursier', period)
        alternant = individu('alternant', period)
        garantie_jeunes = individu('garantie_jeunes', period) > 0

        eligibilite_statut = etudiant_boursier + alternant + garantie_jeunes

        nbptr = individu.foyer_fiscal('nbptr', period.n_2)
        plafond_ressources = parameters(period).prestations_sociales.aides_jeunes.depart1825.plafond_ressources
        plafond_ressources = plafond_ressources.base + 2 * max_(0, nbptr - 1) * plafond_ressources.par_demi_part_supplementaire

        ressources = individu.foyer_fiscal('rfr', period.n_2)

        eligibilite_financiere = ressources <= plafond_ressources

        return eligibilite_age * (eligibilite_statut + eligibilite_financiere)


class depart1825_montant_maximum(Variable):
    value_type = float
    label = 'Montant maximum du dispositif départ 18-25'
    entity = Individu
    definition_period = MONTH
    set_input = set_input_divide_by_period
    reference = [
        'https://programme-depart-1825.com/eligibilite/',
        'https://www.ancv.com/actualites/le-magazine/depart-1825-un-nouveau-programme-pour-les-jeunes-de-18-25-ans'
        ]

    def formula(individu, period, parameters):

        montant = parameters(period).prestations_sociales.aides_jeunes.depart1825.montant_maximum
        eligibilite = individu('depart1825_eligibilite', period)

        return montant * eligibilite



import logging

from openfisca_france.model.base import *
from openfisca_france.model.prelevements_obligatoires.prelevements_sociaux.cotisations_sociales.base import (
    apply_bareme, apply_bareme_for_relevant_type_sal)


log = logging.getLogger(__name__)


# TODO:
# contribution patronale de prévoyance complémentaire
# check hsup everywhere !
# versement transport dépdendant de la localité (décommenter et compléter)


class assiette_cotisations_sociales(Variable):
    value_type = float
    entity = Individu
    label = 'Assiette des cotisations sociales des salaries'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period, parameters):
        assiette_cotisations_sociales_prive = individu('assiette_cotisations_sociales_prive', period)
        remuneration_principale = individu('remuneration_principale', period)
        categorie_salarie = individu('categorie_salarie', period)
        stage_gratification_reintegration = individu('stage_gratification_reintegration', period)
        return (
            (categorie_salarie != TypesCategorieSalarie.non_pertinent)
            * (assiette_cotisations_sociales_prive + remuneration_principale)
            + stage_gratification_reintegration
            )


class assiette_cotisations_sociales_prive(Variable):
    value_type = float
    entity = Individu
    label = 'Assiette des cotisations sociales des salaries du prive'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period, parameters):
        avantage_en_nature = individu('avantage_en_nature', period)
        hsup = individu('hsup', period)
        indemnites_compensatrices_conges_payes = individu('indemnites_compensatrices_conges_payes', period)
        indemnite_residence = individu('indemnite_residence', period)
        primes_fonction_publique = individu('primes_fonction_publique', period)
        primes_salaires_non_exonerees = individu('primes_salaires_non_exonerees', period)
        indemnite_fin_contrat = individu('indemnite_fin_contrat', period)
        reintegration_titre_restaurant_employeur = individu(
            'reintegration_titre_restaurant_employeur', period
            )
        remuneration_apprenti = individu('remuneration_apprenti', period)
        salaire_de_base = individu('salaire_de_base', period)
        categorie_salarie = individu('categorie_salarie', period)

        assiette = (
            salaire_de_base
            + primes_salaires_non_exonerees
            + avantage_en_nature
            + hsup
            + indemnites_compensatrices_conges_payes
            + remuneration_apprenti
            + (categorie_salarie == TypesCategorieSalarie.public_non_titulaire) * (
                indemnite_residence + primes_fonction_publique
                )
            + reintegration_titre_restaurant_employeur
            + indemnite_fin_contrat
            )

        return assiette


class indemnite_fin_contrat(Variable):
    value_type = float
    entity = Individu
    label = 'Indemnité de fin de contrat'
    reference = 'https://www.service-public.fr/particuliers/vosdroits/F40'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period, parameters):
        contrat_de_travail_type = individu('contrat_de_travail_type', period)
        TypesContrat = contrat_de_travail_type.possible_values
        salaire_de_base = individu('salaire_de_base', period)
        categorie_salarie = individu('categorie_salarie', period)
        apprenti = individu('apprenti', period)

        # Un grand nombre de conditions peuvent invalider cette indemnité, voir le lien ci-dessus.
        # A ajouter au fur et à mesure
        # Pour l'instant, cette variable d'entrée peut les remplacer
        # Elle est cependant fixée à False par défaut
        indemnite_fin_contrat_due = individu('indemnite_fin_contrat_due', period)
        taux = parameters(period).prelevements_sociaux.cotisations_securite_sociale_regime_general.indemnite_fin_contrat.taux

        result = (
            # CDD

            (contrat_de_travail_type == TypesContrat.cdd)

            # non fonction publique
            * (
                (categorie_salarie == TypesCategorieSalarie.prive_non_cadre)
                + (categorie_salarie == TypesCategorieSalarie.prive_cadre)
                )
            * not_(apprenti)
            * indemnite_fin_contrat_due
            # 10% du brut
            * taux
            * salaire_de_base
            )

        return result


class reintegration_titre_restaurant_employeur(Variable):
    value_type = float
    entity = Individu
    label = "Prise en charge de l'employeur des dépenses de cantine et des titres restaurants non exonérés de charges sociales"
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period, parameters):
        valeur_unitaire = individu('titre_restaurant_valeur_unitaire', period)
        volume = individu('titre_restaurant_volume', period)
        taux_employeur = individu('titre_restaurant_taux_employeur', period)
        cantines_titres_restaurants = parameters(period).prelevements_sociaux.cotisations_securite_sociale_regime_general.assiette.cantines_titres_restaurants

        taux_minimum_exoneration = cantines_titres_restaurants.taux_minimum_exoneration
        taux_maximum_exoneration = cantines_titres_restaurants.taux_maximum_exoneration
        seuil_prix_titre = cantines_titres_restaurants.seuil_prix_titre

        condition_exoneration_taux = (
            (taux_minimum_exoneration <= taux_employeur)
            * (taux_maximum_exoneration >= taux_employeur)
            )

        montant_reintegration = volume * (
            condition_exoneration_taux
            * max_(valeur_unitaire * taux_employeur - seuil_prix_titre, 0)
            + not_(condition_exoneration_taux)
            * valeur_unitaire
            * taux_employeur
            )

        return montant_reintegration


# Cotisations proprement dites


class penibilite(Variable):
    value_type = float
    entity = Individu
    label = "Les dépenses liées à l'utilisation du compte pénibilité par le salarié sont prises en charge par un fonds financé par l'employeur"
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula_2015_01_01(individu, period, parameters):
        exposition_penibilite = individu('exposition_penibilite', period)
        TypesExpositionPenibilite = exposition_penibilite.possible_values
        multiplicateur = parameters(period).cotsoc.cotisations_employeur.prive_cadre.penibilite_multiplicateur_exposition_multiple

        cotisation_base = apply_bareme(
            individu,
            period,
            parameters,
            cotisation_type = 'employeur',
            bareme_name = 'penibilite_base',
            variable_name = 'penibilite',
            )
        cotisation_additionnelle = apply_bareme(
            individu,
            period,
            parameters,
            cotisation_type = 'employeur',
            bareme_name = 'penibilite_additionnelle',
            variable_name = 'penibilite',
            )

        cotisation = switch(
            exposition_penibilite,
            {
                TypesExpositionPenibilite.nulle: cotisation_base,
                TypesExpositionPenibilite.simple: cotisation_base + cotisation_additionnelle,
                TypesExpositionPenibilite.multiple: cotisation_base + cotisation_additionnelle * multiplicateur,
                }
            )

        return cotisation

    def formula_2018(individu, period, parameters):
        return individu.empty_array()


class accident_du_travail(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisations employeur accident du travail et maladie professionelle'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula_1991(individu, period, parameters):    # TODO : rajouter formule pré-1991 : s'applique au salaire sous PSS uniquement
        assiette_cotisations_sociales = individu(
            'assiette_cotisations_sociales', period)
        taux_accident_travail = individu('taux_accident_travail', period)
        categorie_salarie = individu('categorie_salarie', period)
        assujetti = (
            (categorie_salarie == TypesCategorieSalarie.prive_non_cadre)
            + (categorie_salarie == TypesCategorieSalarie.prive_cadre)
            )
        # TODO: ajouter contractuel du public salarié de moins d'un an ou à temps partiel
        return - assiette_cotisations_sociales * taux_accident_travail * assujetti


class agff_salarie(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisation retraite AGFF tranche A (salarié)'
    definition_period = MONTH
    set_input = set_input_divide_by_period
    end = '2018-12-31'
    # AGFF: Association pour la gestion du fonds de financement (sous-entendu des départs entre 60 et 65 ans)

    def formula(individu, period, parameters):
        cotisation = apply_bareme(
            individu,
            period,
            parameters,
            cotisation_type = 'salarie',
            bareme_name = 'agff',
            variable_name = 'agff_salarie'
            )
        return cotisation


class agff_employeur(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisation retraite AGFF tranche A (employeur)'
    definition_period = MONTH
    set_input = set_input_divide_by_period
    end = '2018-12-31'
    # TODO: améliorer pour gérer mensuel/annuel

    def formula(individu, period, parameters):
        assiette_cotisations_sociales = individu(
            'assiette_cotisations_sociales', period)
        categorie_salarie = individu('categorie_salarie', period)
        plafond_securite_sociale = individu('plafond_securite_sociale', period)

        law = parameters(period)

        cotisation_non_cadre = apply_bareme_for_relevant_type_sal(
            bareme_by_type_sal_name = law.cotsoc.cotisations_employeur,
            bareme_name = 'agffnc',
            base = assiette_cotisations_sociales,
            plafond_securite_sociale = plafond_securite_sociale,
            categorie_salarie = categorie_salarie,
            )

        cotisation_cadre = apply_bareme_for_relevant_type_sal(
            bareme_by_type_sal_name = law.cotsoc.cotisations_employeur,
            bareme_name = 'agffc',
            base = assiette_cotisations_sociales,
            plafond_securite_sociale = plafond_securite_sociale,
            categorie_salarie = categorie_salarie,
            )
        return cotisation_cadre + cotisation_non_cadre


class quotite_de_travail(Variable):
    value_type = float
    entity = Individu
    label = 'Quotité de travail'
    definition_period = MONTH
    set_input = set_input_divide_by_period
    # TODO: gestion annuel/mensuel

    def formula(individu, period, parameters):
        contrat_de_travail = individu('contrat_de_travail', period)
        TypesContratDeTravail = contrat_de_travail.possible_values
        parameters = parameters(period)
        heures_temps_plein = parameters.marche_travail.salaire_minimum.smic.nb_heures_travail_mensuel
        forfait_jours_remuneres_volume = individu('forfait_jours_remuneres_volume', period)
        heures_remunerees_volume = individu('heures_remunerees_volume', period)
        return switch(
            contrat_de_travail,
            {
                TypesContratDeTravail.temps_plein: 1,
                TypesContratDeTravail.temps_partiel: (heures_remunerees_volume / heures_temps_plein),
                TypesContratDeTravail.forfait_jours_annee: (forfait_jours_remuneres_volume / 218),
                TypesContratDeTravail.sans_objet: 0
                }
            )


class agirc_gmp_salarie(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisation AGIRC pour la garantie minimale de points (GMP,  salarié)'
    definition_period = MONTH
    set_input = set_input_divide_by_period
    end = '2018-12-31'
    # TODO: gestion annuel/mensuel

    def formula(individu, period, parameters):
        agirc_salarie = individu('agirc_salarie', period)
        assiette_cotisations_sociales = individu('assiette_cotisations_sociales', period)
        categorie_salarie = individu('categorie_salarie', period)
        quotite = individu('quotite_de_travail', period)

        cadre_cotisant = (
            (categorie_salarie == TypesCategorieSalarie.prive_cadre)
            & (assiette_cotisations_sociales > 0)
            )

        gmp = parameters(period).prelevements_sociaux.regimes_complementaires_retraite_secteur_prive.gmp
        cotisation_forfaitaire_temps_plein = gmp.cotisation_forfaitaire_mensuelle.part_salariale
        cotisation_forfaitaire = cotisation_forfaitaire_temps_plein * quotite

        # Sachant:
        # - qu'il faut retourner un nombre négatif car c'est un prélèvement,
        # - que la cotisation agirc_salarie est négative car c'est un prélèvement,
        # - que la cotisation_forfaitaire est positive,
        # le montant de la gmp est cotisation_forfaitaire - (-agirc_salarie) soit:
        return - max_(cotisation_forfaitaire + agirc_salarie, 0) * cadre_cotisant


class agirc_gmp_employeur(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisation AGIRC pour la garantie minimale de points (GMP, employeur)'
    definition_period = MONTH
    set_input = set_input_divide_by_period
    end = '2018-12-31'
    # TODO: gestion annuel/mensuel

    def formula(individu, period, parameters):
        agirc_employeur = individu('agirc_employeur', period)
        assiette_cotisations_sociales = individu('assiette_cotisations_sociales', period)
        categorie_salarie = individu('categorie_salarie', period)
        quotite = individu('quotite_de_travail', period)

        cadre_cotisant = (
            (categorie_salarie == TypesCategorieSalarie.prive_cadre)
            & (assiette_cotisations_sociales > 0)
            )

        gmp = parameters(period).prelevements_sociaux.regimes_complementaires_retraite_secteur_prive.gmp
        cotisation_forfaitaire_temps_plein = gmp.cotisation_forfaitaire_mensuelle.part_patronale
        cotisation_forfaitaire = cotisation_forfaitaire_temps_plein * quotite

        # Sachant:
        # - qu'il faut retourner un nombre négatif car c'est un prélèvement,
        # - que la cotisation agirc_salarie est négative car c'est un prélèvement,
        # - que la cotisation_forfaitaire est positive,
        # le montant de la gmp est cotisation_forfaitaire - (-agirc_salarie) soit:
        return - max_(cotisation_forfaitaire + agirc_employeur, 0) * cadre_cotisant


class agirc_salarie(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisation AGIRC tranche B (salarié)'
    definition_period = MONTH
    set_input = set_input_divide_by_period
    end = '2018-12-31'

    def formula(individu, period, parameters):
        cotisation = apply_bareme(
            individu,
            period,
            parameters,
            cotisation_type = 'salarie',
            bareme_name = 'agirc',
            variable_name = 'agirc_salarie'
            )
        categorie_salarie = individu('categorie_salarie', period)
        return cotisation * (categorie_salarie == TypesCategorieSalarie.prive_cadre)


class agirc_employeur(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisation AGIRC tranche B (employeur)'
    definition_period = MONTH
    set_input = set_input_divide_by_period
    end = '2018-12-31'

    def formula(individu, period, parameters):
        cotisation = apply_bareme(
            individu,
            period,
            parameters,
            cotisation_type = 'employeur',
            bareme_name = 'agirc',
            variable_name = 'agirc_employeur'
            )
        categorie_salarie = individu('categorie_salarie', period)
        return cotisation * (categorie_salarie == TypesCategorieSalarie.prive_cadre)


class agirc_arrco_salarie(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisation AGIRC-ARRCO (après la fusion, salarié)'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula_2019_01_01(individu, period, parameters):
        cotisation = apply_bareme(
            individu,
            period,
            parameters,
            cotisation_type = 'salarie',
            bareme_name = 'agirc_arrco',
            variable_name = 'agirc_arrco_salarie'
            )
        return cotisation


class agirc_arrco_employeur(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisation AGIRC-ARRCO (après la fusion, employeur)'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula_2019_01_01(individu, period, parameters):
        cotisation = apply_bareme(
            individu,
            period,
            parameters,
            cotisation_type = 'employeur',
            bareme_name = 'agirc_arrco',
            variable_name = 'agirc_arrco_employeur'
            )
        return cotisation


class ags(Variable):
    value_type = float
    entity = Individu
    label = "Contribution à l'association pour la gestion du régime de garantie des créances des salariés (AGS, employeur)"
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period, parameters):
        cotisation = apply_bareme(
            individu,
            period,
            parameters,
            cotisation_type = 'employeur',
            bareme_name = 'ags',
            variable_name = 'ags',
            )
        return cotisation


class apec_salarie(Variable):
    value_type = float
    entity = Individu
    label = "Cotisations agence pour l'emploi des cadres (APEC,  salarié)"
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period, parameters):
        categorie_salarie = individu('categorie_salarie', period)
        cotisation = apply_bareme(
            individu,
            period,
            parameters,
            cotisation_type = 'salarie',
            bareme_name = 'apec',
            variable_name = 'apec_salarie',
            )
        return cotisation * (categorie_salarie == TypesCategorieSalarie.prive_cadre)  # TODO: check public notamment contractuel


class apec_employeur(Variable):
    value_type = float
    entity = Individu
    label = "Cotisations Agenece pour l'emploi des cadres (APEC, employeur)"
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period, parameters):
        cotisation = apply_bareme(
            individu,
            period,
            parameters,
            cotisation_type = 'employeur',
            bareme_name = 'apec',
            variable_name = 'apec_employeur',
            )
        return cotisation  # TODO: check public notamment contractuel


class arrco_salarie(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisation ARRCO tranche 1 (salarié)'
    definition_period = MONTH
    set_input = set_input_divide_by_period
    end = '2018-12-31'
    # TODO: check gestion mensuel/annuel

    def formula(individu, period, parameters):
        cotisation_minimale = apply_bareme(
            individu,
            period,
            parameters,
            cotisation_type = 'salarie',
            bareme_name = 'arrco',
            variable_name = 'arrco_salarie',
            )
        arrco_tranche_a_taux_salarie = individu('arrco_tranche_a_taux_salarie', period)
        assiette_cotisations_sociales = individu('assiette_cotisations_sociales', period, options = [ADD])
        plafond_securite_sociale = individu('plafond_securite_sociale', period, options = [ADD])
        categorie_salarie = individu('categorie_salarie', period)

        # cas où l'entreprise applique un taux spécifique
        cotisation_entreprise = - (
            min_(max_(assiette_cotisations_sociales, 0), plafond_securite_sociale)
            * arrco_tranche_a_taux_salarie
            )

        public = (
            (categorie_salarie == TypesCategorieSalarie.prive_non_cadre)
            + (categorie_salarie == TypesCategorieSalarie.prive_cadre)
            )

        return (
            cotisation_minimale
            * (arrco_tranche_a_taux_salarie == 0)
            + cotisation_entreprise
            ) * public


class arrco_employeur(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisation ARRCO tranche 1 (employeur)'
    definition_period = MONTH
    set_input = set_input_divide_by_period
    end = '2018-12-31'
    # TODO: check gestion mensuel/annuel

    def formula(individu, period, parameters):
        cotisation_minimale = apply_bareme(
            individu,
            period,
            parameters,
            cotisation_type = 'employeur',
            bareme_name = 'arrco',
            variable_name = 'arrco_employeur',
            )
        arrco_tranche_a_taux_employeur = individu('arrco_tranche_a_taux_employeur', period)
        assiette_cotisations_sociales = individu('assiette_cotisations_sociales', period, options = [ADD])
        plafond_securite_sociale = individu('plafond_securite_sociale', period, options = [ADD])
        categorie_salarie = individu('categorie_salarie', period)

        # cas où l'entreprise applique un taux spécifique
        cotisation_entreprise = - (
            min_(max_(assiette_cotisations_sociales, 0), plafond_securite_sociale)
            * arrco_tranche_a_taux_employeur
            )

        public = (
            (categorie_salarie == TypesCategorieSalarie.prive_non_cadre)
            + (categorie_salarie == TypesCategorieSalarie.prive_cadre)
            )
        return (
            cotisation_minimale * (arrco_tranche_a_taux_employeur == 0) + cotisation_entreprise
            ) * public


class chomage_salarie(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisation chômage (salarié)'
    definition_period = MONTH
    set_input = set_input_divide_by_period
    end = '2018-09-30'

    def formula(individu, period, parameters):
        cotisation_chomage = apply_bareme(
            individu,
            period,
            parameters,
            cotisation_type = 'salarie',
            bareme_name = 'chomage',
            variable_name = 'chomage_salarie',
            )
        asf = apply_bareme(
            individu,
            period,
            parameters,
            cotisation_type = 'salarie',
            bareme_name = 'asf',
            variable_name = 'chomage_salarie',
            )

        return cotisation_chomage + asf


class chomage_employeur(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisation chômage (employeur)'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period, parameters):
        cotisation_chomage = apply_bareme(
            individu,
            period,
            parameters,
            cotisation_type = 'employeur',
            bareme_name = 'chomage',
            variable_name = 'chomage_employeur',
            )
        asf = apply_bareme(
            individu,
            period,
            parameters,
            cotisation_type = 'employeur',
            bareme_name = 'asf',
            variable_name = 'chomage_salarie',
            )

        return cotisation_chomage + asf


class contribution_equilibre_general_salarie(Variable):
    value_type = float
    entity = Individu
    label = "Contribution d'équilibre général (salarie)"
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula_2019_01_01(individu, period, parameters):

        cotisation = apply_bareme(
            individu,
            period,
            parameters,
            cotisation_type = 'salarie',
            bareme_name = 'ceg',
            variable_name = 'contribution_equilibre_general_salarie',
            )
        return cotisation


class contribution_equilibre_general_employeur(Variable):
    value_type = float
    entity = Individu
    label = "Contribution d'équilibre général (employeur)"
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula_2019_01_01(individu, period, parameters):

        cotisation = apply_bareme(
            individu,
            period,
            parameters,
            cotisation_type = 'employeur',
            bareme_name = 'ceg',
            variable_name = 'contribution_equilibre_general_employeur',
            )
        return cotisation


class contribution_solidarite_autonomie(Variable):
    value_type = float
    entity = Individu
    label = 'Contribution solidarité autonomie (employeur)'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period, parameters):
        cotisation = apply_bareme(
            individu,
            period,
            parameters,
            cotisation_type = 'employeur',
            bareme_name = 'csa',
            variable_name = 'contribution_solidarite_autonomie',
            )
        return cotisation


class contribution_equilibre_technique_salarie(Variable):
    value_type = float
    entity = Individu
    label = "Contribution d'équilibre technique (salarie)"
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula_2019_01_01(individu, period, parameters):

        plafond_securite_sociale = individu('plafond_securite_sociale', period)
        assiette_cotisations_sociales = individu('assiette_cotisations_sociales', period)

        cotisation = apply_bareme(
            individu,
            period,
            parameters,
            cotisation_type = 'salarie',
            bareme_name = 'cet2019',
            variable_name = 'contribution_equilibre_technique_salarie',
            )
        return cotisation * (assiette_cotisations_sociales > plafond_securite_sociale)


class contribution_equilibre_technique_employeur(Variable):
    value_type = float
    entity = Individu
    label = "Contribution d'équilibre technique (employeur)"
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula_2019_01_01(individu, period, parameters):

        plafond_securite_sociale = individu('plafond_securite_sociale', period)
        assiette_cotisations_sociales = individu('assiette_cotisations_sociales', period)

        cotisation = apply_bareme(
            individu,
            period,
            parameters,
            cotisation_type = 'employeur',
            bareme_name = 'cet2019',
            variable_name = 'contribution_equilibre_technique_employeur',
            )
        return cotisation * (assiette_cotisations_sociales > plafond_securite_sociale)


class cotisation_exceptionnelle_temporaire_salarie(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisation_exceptionnelle_temporaire (salarie)'
    definition_period = MONTH
    set_input = set_input_divide_by_period
    end = '2018-12-31'

    def formula(individu, period, parameters):
        cotisation = apply_bareme(
            individu,
            period,
            parameters,
            cotisation_type = 'salarie',
            bareme_name = 'cet',
            variable_name = 'cotisation_exceptionnelle_temporaire_salarie',
            )
        return cotisation


class cotisation_exceptionnelle_temporaire_employeur(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisation exceptionnelle temporaire (employeur)'
    definition_period = MONTH
    set_input = set_input_divide_by_period
    end = '2018-12-31'

    def formula(individu, period, parameters):
        cotisation = apply_bareme(
            individu,
            period,
            parameters,
            cotisation_type = 'employeur',
            bareme_name = 'cet',
            variable_name = 'cotisation_exceptionnelle_temporaire_employeur',
            )
        return cotisation


class famille(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisation famille (employeur)'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    # L'allègement général de cotisations de 2015 n'est pas dans le barème mais pris en compte dans
    # allegements.allegement_cotisation_allocations_familiales,
    # (sur la base du paramètre prelevements_sociaux/reductions_cotisations_sociales/allegement_cotisation_allocations_familiales/reduction)
    def formula(individu, period, parameters):
        cotisation = apply_bareme(
            individu,
            period,
            parameters,
            cotisation_type = 'employeur',
            bareme_name = 'famille',
            variable_name = 'famille',
            )
        return cotisation


class mmid_salarie(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisation maladie (salarié)'
    reference = 'https://www.legifrance.gouv.fr/codes/article_lc/LEGIARTI000036390317/ et https://www.legifrance.gouv.fr/codes/article_lc/LEGIARTI000036679615'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula_2018(individu, period, parameters):
        '''
        La cotisation maladie (hors Alsace-Moselle) disparaît à partir du 1er janvier 2018
        '''
        salarie_regime_alsace_moselle = individu('salarie_regime_alsace_moselle', period)
        cotisation_regime_alsace_moselle = apply_bareme(
            individu,
            period,
            parameters,
            cotisation_type = 'salarie',
            bareme_name = 'maladie_alsace_moselle',
            variable_name = 'mmid_salarie',
            )

        cotisation = salarie_regime_alsace_moselle * cotisation_regime_alsace_moselle

        return cotisation

    def formula(individu, period, parameters):
        salarie_regime_alsace_moselle = individu('salarie_regime_alsace_moselle', period)

        cotisation_regime_general = apply_bareme(
            individu,
            period,
            parameters,
            cotisation_type = 'salarie',
            bareme_name = 'maladie',
            variable_name = 'mmid_salarie',
            )

        cotisation_regime_alsace_moselle = apply_bareme(
            individu,
            period,
            parameters,
            cotisation_type = 'salarie',
            bareme_name = 'maladie_alsace_moselle',
            variable_name = 'mmid_salarie',
            )

        cotisation = cotisation_regime_general + salarie_regime_alsace_moselle * cotisation_regime_alsace_moselle

        return cotisation


class mmid_employeur(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisation maladie (employeur)'
    reference = 'https://www.urssaf.fr/portail/home/employeur/calculer-les-cotisations/les-taux-de-cotisations/la-cotisation-maladie---maternit.html'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period, parameters):
        cotisation = apply_bareme(
            individu,
            period,
            parameters,
            cotisation_type = 'employeur',
            bareme_name = 'maladie',
            variable_name = 'mmid_employeur',
            )
        return cotisation


class mmida_employeur(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisation maladie (employeur)'
    definition_period = MONTH
    set_input = set_input_divide_by_period
    # Note: this formula is used only to check fiche_de_paie from memento

    def formula(individu, period, parameters):
        cotisation = apply_bareme(
            individu,
            period,
            parameters,
            cotisation_type = 'employeur',
            bareme_name = 'maladie',
            variable_name = 'mmida_employeur',
            )
        contribution_solidarite_autonomie = individu('contribution_solidarite_autonomie', period)
        return cotisation + contribution_solidarite_autonomie


class plafond_securite_sociale(Variable):
    value_type = float
    entity = Individu
    label = 'Plafond de la sécurite sociale'
    definition_period = MONTH
    set_input = set_input_divide_by_period
    # TODO gérer les plafonds mensuel, trimestriel, annuel

    def formula(individu, period, parameters):
        plafond_temps_plein = parameters(period).prelevements_sociaux.pss.plafond_securite_sociale_mensuel
        quotite = individu('quotite_de_travail', period)

        plafond = plafond_temps_plein * quotite

        # 2) Proratisation pour mois incomplet selon la méthode des 30èmes

        # Pour les salariés entrés ou sortis en cours de mois,
        # le plafond applicable est égal à autant de trentièmes du plafond mensuel
        # que le salarié a été présent de jours calendaires. Source urssaf.fr "L’assiette maximale"
        # calcul du nombre de jours calendaires de présence du salarié
        nombre_jours_calendaires = individu('nombre_jours_calendaires', period)
        plafond = plafond * (min_(nombre_jours_calendaires, 30) / 30)

        return plafond


class prevoyance_obligatoire_cadre(Variable):
    value_type = float
    entity = Individu
    label = 'Contribution de prévoyance obligatoire pour les cadres et assimilés'
    definition_period = MONTH
    set_input = set_input_divide_by_period
    '''
    La prévoyance obligatoire des cadres n'est pas soumise à CSG/CRDS ni à cotisations sociales.
    '''
    # TODO: gérer le mode de recouvrement et l'aspect mensuel/annuel

    def formula(individu, period, parameters):
        categorie_salarie = individu('categorie_salarie', period)
        assiette_cotisations_sociales = individu('assiette_cotisations_sociales', period)
        plafond_securite_sociale = individu('plafond_securite_sociale', period)
        prevoyance_obligatoire_cadre_taux_employeur = individu(
            'prevoyance_obligatoire_cadre_taux_employeur', period)
        minimum = parameters(period).prelevements_sociaux.autres_taxes_participations_assises_salaires.prevoyance.prevoyance_obligatoire
        taux = max_(minimum, prevoyance_obligatoire_cadre_taux_employeur)

        cotisation = - (
            (categorie_salarie == TypesCategorieSalarie.prive_cadre)
            * min_(assiette_cotisations_sociales, plafond_securite_sociale)
            * taux
            )
        return cotisation


class prevoyance_complementaire_employeur(Variable):
    value_type = float
    entity = Individu
    label = 'Contributions de prévoyance complémentaire'
    definition_period = MONTH
    set_input = set_input_divide_by_period
    '''
    On définit la prévoyance complémentaire par référence à la prévoyance obligatoire : c'est la partie non obligatoire des contributions de prévoyance totales versées par l'employeur.
    La prévoyance complémentaire est assujettie à CSG et au forfait social, ce qui n'est pas le cas de la prévoyance obligatoire.
    '''

    def formula(individu, period):
        prevoyance_totale = individu('prevoyance_employeur', period)
        prevoyance_obligatoire = individu('prevoyance_obligatoire_cadre', period)
        cotisation = - max_(
            0,
            prevoyance_totale
            - (-prevoyance_obligatoire)
            )

        return cotisation


class complementaire_sante_employeur(Variable):
    value_type = float
    entity = Individu
    label = "Couverture complémentaire santé collective d'entreprise - part employeur"
    reference = 'https://www.legifrance.gouv.fr/codes/article_lc/LEGIARTI000038610242'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula_2016_01_01(individu, period, parameters):
        complementaire_sante_part_employeur = individu('complementaire_sante_part_employeur', period)
        minimum = parameters(period).prelevements_sociaux.autres_taxes_participations_assises_salaires.complementaire_sante.part_employeur
        part_employeur = max_(complementaire_sante_part_employeur, minimum)
        complementaire_sante_montant = individu('complementaire_sante_montant', period)
        cotisation = - part_employeur * complementaire_sante_montant

        return cotisation

    def formula(individu, period):
        part_employeur = individu('complementaire_sante_part_employeur', period)
        complementaire_sante_montant = individu('complementaire_sante_montant', period)
        cotisation = - part_employeur * complementaire_sante_montant

        return cotisation


class complementaire_sante_salarie(Variable):
    value_type = float
    entity = Individu
    label = "Couverture complémentaire santé collective d'entreprise - part salarié"
    reference = 'https://www.legifrance.gouv.fr/codes/article_lc/LEGIARTI000038610242'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula_2016_01_01(individu, period, parameters):
        complementaire_sante_part_employeur = individu('complementaire_sante_part_employeur', period)
        minimum = parameters(period).prelevements_sociaux.autres_taxes_participations_assises_salaires.complementaire_sante.part_employeur
        part_employeur = max_(complementaire_sante_part_employeur, minimum)
        complementaire_sante_montant = individu('complementaire_sante_montant', period)
        cotisation = - (1 - part_employeur) * complementaire_sante_montant

        return cotisation

    def formula(individu, period):
        part_employeur = individu('complementaire_sante_part_employeur', period)
        complementaire_sante_montant = individu('complementaire_sante_montant', period)
        cotisation = - (1 - part_employeur) * complementaire_sante_montant

        return cotisation


class TypesTailleEntreprise(Enum):
    __order__ = 'non_pertinent moins_de_10 de_10_a_19 de_20_a_249 plus_de_250'  # Needed to preserve the enum order in Python 2
    non_pertinent = 'Non pertinent'
    moins_de_10 = 'Moins de 10 salariés'
    de_10_a_19 = 'De 10 à 19 salariés'
    de_20_a_249 = 'De 20 à 249 salariés'
    plus_de_250 = 'Plus de 250 salariés'


class taille_entreprise(Variable):
    value_type = Enum
    possible_values = TypesTailleEntreprise
    default_value = TypesTailleEntreprise.non_pertinent
    entity = Individu
    label = "Catégorie de taille d'entreprise"
    reference = 'http://www.insee.fr/fr/themes/document.asp?ref_id=ip1321'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(individu, period, parameters):
        effectif_entreprise = individu('effectif_entreprise', period)

        taille_entreprise = select(
            [
                (effectif_entreprise <= 0),
                (effectif_entreprise <= 9),
                (effectif_entreprise <= 19),
                (effectif_entreprise <= 249),
                (effectif_entreprise >= 250)
                ],
            [
                TypesTailleEntreprise.non_pertinent,
                TypesTailleEntreprise.moins_de_10,
                TypesTailleEntreprise.de_10_a_19,
                TypesTailleEntreprise.de_20_a_249,
                TypesTailleEntreprise.plus_de_250
                ]
            )
        return taille_entreprise


class taux_accident_travail(Variable):
    value_type = float
    entity = Individu
    label = "Approximation du taux accident à partir de l'exposition au risque donnée"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula_2012_01_01(individu, period, parameters):
        exposition_accident = individu('exposition_accident', period)
        TypesExpositionAccident = exposition_accident.possible_values
        accident = parameters(period).prelevements_sociaux.cotisations_securite_sociale_regime_general.accidents.taux
        return select(
            [
                exposition_accident == TypesExpositionAccident.faible,
                exposition_accident == TypesExpositionAccident.moyen,
                exposition_accident == TypesExpositionAccident.eleve,
                exposition_accident == TypesExpositionAccident.tres_eleve,
                ],
            [
                accident.faible,
                accident.moyen,
                accident.eleve,
                accident.treseleve
                ]
            )


class vieillesse_deplafonnee_salarie(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisation vieillesse déplafonnée (salarié)'
    reference = [
        'Article L. 242-1 du code de la sécurité sociale',
        'https://www.legifrance.gouv.fr/codes/article_lc/LEGIARTI000044626664'
        ]
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period, parameters):
        cotisation = apply_bareme(
            individu,
            period,
            parameters,
            cotisation_type = 'salarie',
            bareme_name = 'vieillesse_deplafonnee',
            variable_name = 'vieillesse_deplafonnee_salarie',
            )
        return cotisation


class vieillesse_plafonnee_salarie(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisation vieillesse plafonnée (salarié)'
    reference = [
        'Article L. 242-1 du code de la sécurité sociale',
        'https://www.legifrance.gouv.fr/codes/article_lc/LEGIARTI000044626664'
        ]
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period, parameters):
        cotisation = apply_bareme(
            individu,
            period,
            parameters,
            cotisation_type = 'salarie',
            bareme_name = 'vieillesse_plafonnee',
            variable_name = 'vieillesse_plafonnee_salarie',
            )
        return cotisation


class vieillesse_deplafonnee_employeur(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisation vieillesse déplafonnée'
    definition_period = MONTH
    set_input = set_input_divide_by_period
    reference = [
        'Article L. 242-1 du code de la sécurité sociale',
        'https://www.legifrance.gouv.fr/codes/article_lc/LEGIARTI000044626664'
        ]

    def formula(individu, period, parameters):
        cotisation = apply_bareme(
            individu,
            period,
            parameters,
            cotisation_type = 'employeur',
            bareme_name = 'vieillesse_deplafonnee',
            variable_name = 'vieillesse_deplafonnee_employeur',
            )
        return cotisation


class vieillesse_plafonnee_employeur(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisation vieillesse plafonnée (employeur)'
    definition_period = MONTH
    set_input = set_input_divide_by_period
    reference = [
        'Article L. 242-1 du code de la sécurité sociale',
        'https://www.legifrance.gouv.fr/codes/article_lc/LEGIARTI000044626664'
        ]

    def formula(individu, period, parameters):
        cotisation = apply_bareme(
            individu,
            period,
            parameters,
            cotisation_type = 'employeur',
            bareme_name = 'vieillesse_plafonnee',
            variable_name = 'vieillesse_plafonnee_employeur',
            )
        return cotisation


class peec_employeur(Variable):
    value_type = bool
    entity = Individu
    label = "Participation de l'employeur à l'effort de construction (PEEC) communément appelé le 1% logement (employeur)"
    definition_period = MONTH
    set_input = set_input_divide_by_period
    reference = 'https://www.service-public.fr/professionnels-entreprises/vosdroits/F22583'


class secteur_activite_employeur(Variable):
    value_type = Enum
    possible_values = TypesSecteurActivite
    default_value = TypesSecteurActivite.non_renseigne
    entity = Individu
    label = "Secteur d'activité (employeur)"
    definition_period = MONTH
    reference = 'https://www.service-public.fr/professionnels-entreprises/vosdroits/N24269'
    set_input = set_input_dispatch_by_period



import os

from ..model.base import *


dir_path = os.path.join(os.path.dirname(__file__), 'parameters')


def modify_parameters(parameters):
    reform_year = 2013
    reform_period = period(reform_year)

    file_path = os.path.join(dir_path, 'plf2015.yaml')
    reform_parameters_subtree = load_parameter_file(name='plf2015', file_path=file_path)
    parameters.add_child('plf2015', reform_parameters_subtree)

    parameters.impot_revenu.bareme_ir_depuis_1945.bareme[1].rate.update(period=reform_period, value=0)
    parameters.impot_revenu.bareme_ir_depuis_1945.bareme[2].threshold.update(period=reform_period, value=9690)

    return parameters


class decote(Variable):
    label = 'Décote IR 2015 appliquée sur IR 2014 (revenus 2013)'
    definition_period = YEAR

    def formula_2013_01_01(foyer_fiscal, period, parameters):
        ir_plaf_qf = foyer_fiscal('ir_plaf_qf', period)
        nb_adult = foyer_fiscal('nb_adult', period)
        plf = parameters(period).plf2015

        decote_celib = (ir_plaf_qf < plf.seuil_celib) * (plf.seuil_celib - ir_plaf_qf)
        decote_couple = (ir_plaf_qf < plf.seuil_couple) * (plf.seuil_couple - ir_plaf_qf)
        return (nb_adult == 1) * decote_celib + (nb_adult == 2) * decote_couple


class plf2015(Reform):
    name = 'Projet de Loi de Finances 2015 appliquée aux revenus 2013'

    def apply(self):
        self.update_variable(decote)
        self.modify_parameters(modifier_function = modify_parameters)



from openfisca_france.model.base import *


class asf_elig_enfant(Variable):
    value_type = bool
    entity = Individu
    label = "Enfant pouvant ouvrir droit à l'ASF"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(individu, period, parameters):
        age = individu('age', period)
        autonomie_financiere = individu('autonomie_financiere', period)

        af = parameters(period).prestations_sociales.prestations_familiales.prestations_generales.af

        eligibilite = (
            # Âge compatible avec les prestations familiales
            (age >= af.af_cm.age1)
            * (age < af.af_cm.age3)
            * not_(autonomie_financiere)  # Ne perçoit pas plus de ressources que "55% du SMIC" au sens CAF
            )

        return eligibilite


class asf_elig(Variable):
    value_type = bool
    entity = Famille
    label = "Éligibilité à l'ASF"
    reference = ['https://www.aide-sociale.fr/allocation-soutien-familial/']
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(famille, period):
        isole = not_(famille('en_couple', period))
        residence_mayotte = famille.demandeur.menage('residence_mayotte', period)

        # Parent isolé et ne résident pas à Mayotte
        return not_(residence_mayotte) * isole


class asf_montant(Variable):
    calculate_output = calculate_output_add
    value_type = float
    entity = Famille
    label = "Montant de l'allocation de soutien familial (ASF)"
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(famille, period, parameters):
        bmaf = parameters(period).prestations_sociales.prestations_familiales.bmaf.bmaf
        asf = parameters(period).prestations_sociales.prestations_familiales.education_presence_parentale.asf

        asf_par_enfant = (
            famille.members('asf_elig_enfant', period)
            * bmaf
            * asf.montant_asf.orphelin_assimile_seul_parent
            )

        montant = famille.sum(asf_par_enfant, role = Famille.ENFANT)
        pensions_alimentaires_percues = famille.sum(famille.members('pensions_alimentaires_percues', period))

        return max_(montant - pensions_alimentaires_percues, 0)


class asf(Variable):
    calculate_output = calculate_output_add
    value_type = float
    entity = Famille
    label = 'Allocation de soutien familial (ASF)'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(famille, period, parameters):
        asf = parameters(period).prestations_sociales.prestations_familiales.education_presence_parentale.asf

        asf_elig = famille('asf_elig', period)
        montant = famille('asf_montant', period)

        return asf_elig * (montant > asf.seuil) * montant



'''Basic test of all covered years.'''

import datetime

from openfisca_france.scenarios import init_single_entity

from .cache import tax_benefit_system

import pytest


scenarios_arguments = [
    dict(
        period = year,
        parent1 = dict(
            date_naissance = datetime.date(1972, 1, 1),
            salaire_de_base = 2000,
            effectif_entreprise = 25,
            categorie_salarie = 'prive_non_cadre',
            ),
        parent2 = dict(
            date_naissance = datetime.date(1972, 1, 1),
            ),
        menage = dict(
            zone_apl = 'zone_1',
            ),
        )
    for year in range(2006, 2020)
    ]


@pytest.mark.parametrize('scenario_arguments', scenarios_arguments)
def test_basics(scenario_arguments):
    '''Basic test for a specific year.

    Args:
        scenario_arguments (dict): Arguments to initialize scenario.
    '''
    scenario = tax_benefit_system.new_scenario()
    init_single_entity(scenario, **scenario_arguments)
    simulation = scenario.new_simulation(debug = False)
    period = scenario_arguments['period']
    assert simulation.calculate('revenu_disponible', period = period) is not None, "Can't compute revenu_disponible on period {}".format(period)
    assert simulation.calculate_add('salaire_super_brut', period = period) is not None, "Can't compute salaire_super_brut on period {}".format(period)



from openfisca_core import periods

from openfisca_france.model.base import *

from numpy import logical_or as or_, logical_and as and_

# Les éligibilités séparées de l'indemnité inflation
#######################################################


# 1 : Non-Salariés
class eligibilite_indemnite_inflation_non_salarie(Variable):
    entity = Individu
    value_type = bool
    reference = 'https://www.legifrance.gouv.fr/jorf/id/JORFTEXT000044471405'
    label = "Eligibilité à l'indemnité inflation en tant que non-salarié"
    definition_period = YEAR

    def formula(individu, period, parameters):

        oct_2021 = periods.period('2021-10')

        # non-salarié
        eligibilite_cat_non_sal = (individu('categorie_non_salarie', oct_2021.this_year) != TypesCategorieNonSalarie.non_pertinent)

        # revenu d'activité inférieure à € 2000 nets par mois en 2020 (selon déclaration annuelle des revenus)
        annee_2020 = periods.period('2020')
        jan_sep_2021 = periods.period('month:2021-01:9')

        # chiffre d'affaires. Uniquement pour les régimes microsociaux, on ne tient pas compte de ceux qui y auraient droit (et à l'indemnite inflation) mais préféreraient le régime des bénéfices réels.
        rev_net_auto = individu('rpns_auto_entrepreneur_revenus_net', annee_2020, options = [ADD])
        rev_net_micro = individu('rpns_micro_entreprise_revenus_net', annee_2020, options = [ADD])

        rev_net = (rev_net_auto + rev_net_micro) / 12

        chiffre_d_affaires_micro = individu('rpns_micro_entreprise_chiffre_affaires', jan_sep_2021.this_year) * 9 / 12
        chiffre_d_affaires_liberatoire = individu('rpns_auto_entrepreneur_chiffre_affaires', jan_sep_2021.this_year) * 9 / 12
        chiffre_d_affaires = chiffre_d_affaires_micro + chiffre_d_affaires_liberatoire

        eligibilite_micro_artisan = and_(and_(chiffre_d_affaires >= 900, chiffre_d_affaires <= 4000), individu('categorie_non_salarie', oct_2021.this_year) == TypesCategorieNonSalarie.artisan)
        eligibilite_micro_commercant = and_(and_(chiffre_d_affaires >= 900, chiffre_d_affaires <= 6897), individu('categorie_non_salarie', oct_2021.this_year) == TypesCategorieNonSalarie.commercant)
        eligibilite_micro_prof_lib = and_(and_(chiffre_d_affaires >= 900, chiffre_d_affaires <= 3030), individu('categorie_non_salarie', oct_2021.this_year) == TypesCategorieNonSalarie.profession_liberale)

        eligibilite_micro = (eligibilite_micro_artisan + eligibilite_micro_commercant + eligibilite_micro_prof_lib) > 0

        return eligibilite_cat_non_sal * or_(and_(rev_net <= 2000, rev_net > 0),
                                             eligibilite_micro)


# 2 : Salariés
class eligibilite_indemnite_inflation_salarie_prive(Variable):
    entity = Individu
    value_type = bool
    reference = 'https://www.legifrance.gouv.fr/jorf/id/JORFTEXT000044471405'
    label = "Eligibilité à l'indemnité inflation en tant que salarié privé"
    definition_period = YEAR
    set_input = set_input_dispatch_by_period

    def formula(individu, period, parameters):

        # éligibilité statut
        oct_2021 = periods.period('2021-10')
        eligibilite_activite = individu('activite', oct_2021) == TypesActivite.actif
        eligibilite_alternance = individu('alternant', oct_2021) > 0
        eligibilite = (eligibilite_activite + eligibilite_alternance) > 0

        # éligibilité salaire
        # rémunération moyenne inférieure à € 2000 nets par mois travaillé avant IR
        # ou brut inférieure à € 2600 par mois
        # du janvier à octobre 2021
        rev_periods = ['2021-01',
        '2021-02',
        '2021-03',
        '2021-04',
        '2021-05',
        '2021-06',
        '2021-07',
        '2021-08',
        '2021-09',
        '2021-10']

        nsal = 0
        tsal = 0

        for rp in rev_periods:
            per = periods.period(rp)
            sal = individu('salaire_net', per, options=[ADD])

            if sal > 0:
                nsal += 1
                tsal += sal

        sal_plafond = 2000 * max(1, nsal)

        elig_sal = tsal <= sal_plafond

        # pas non-salarié :
        pas_autre = individu('eligibilite_indemnite_inflation_non_salarie', period) == 0

        return eligibilite * elig_sal * pas_autre


# 3 : Public
class eligibilite_indemnite_inflation_public(Variable):
    entity = Individu
    value_type = bool
    reference = 'https://www.legifrance.gouv.fr/jorf/id/JORFTEXT000044471405'
    label = "Eligibilité à l'indemnité inflation en tant qu'agent public"
    definition_period = YEAR

    def formula(individu, period, parameters):

        oct_2021 = periods.period('2021-10')

        # agent public
        eligibilite_public = ((individu('categorie_salarie', oct_2021) == TypesCategorieSalarie.public_titulaire_etat)
                            + (individu('categorie_salarie', oct_2021) == TypesCategorieSalarie.public_titulaire_militaire)
                            + (individu('categorie_salarie', oct_2021) == TypesCategorieSalarie.public_titulaire_territoriale)
                            + (individu('categorie_salarie', oct_2021) == TypesCategorieSalarie.public_titulaire_hospitaliere)
                            + (individu('categorie_salarie', oct_2021) == TypesCategorieSalarie.public_non_titulaire)) > 0

        # éligibilité rémuneration
        # rémunération moyenne inférieure à € 2000 nets par mois travaillé avant IR
        # ou brut inférieure à € 2600 par mois
        # du janvier à octobre 2021
        rev_periods = ['2021-01',
        '2021-02',
        '2021-03',
        '2021-04',
        '2021-05',
        '2021-06',
        '2021-07',
        '2021-08',
        '2021-09',
        '2021-10']

        nsal = 0
        tsal = 0

        for rp in rev_periods:
            per = periods.period(rp)
            sal = individu('salaire_net', per, options=[ADD])

            if sal > 0:
                nsal += 1
                tsal += sal

        sal_plafond = 2000 * max(1, nsal)

        elig_sal = tsal <= sal_plafond

        # pas non-salarié, salarié :
        pas_autre = (individu('eligibilite_indemnite_inflation_non_salarie', period)
        + individu('eligibilite_indemnite_inflation_salarie_prive', period)) == 0

        return eligibilite_public * elig_sal * pas_autre


# 4 : Retraité
class eligibilite_indemnite_inflation_retraite(Variable):
    entity = Individu
    value_type = bool
    reference = 'https://www.legifrance.gouv.fr/jorf/id/JORFTEXT000044471405'
    label = "Eligibilité à l'indemnité inflation en tant que retraité"
    definition_period = YEAR

    def formula(individu, period, parameters):

        oct_2021 = periods.period('2021-10')
        annee_2021 = periods.period('2021')

        # bénéficiaire d'une pension de retraite en octobre 2021
        eligibilite_retraite = (individu('activite', oct_2021) == TypesActivite.retraite)

        # bénéficiaire du minimum vieillesse en octobre 2021
        eligibilite_aspa = (individu.famille('aspa', oct_2021) > 0)

        # pension/aspa nette inférieur à € 2000 par mois
        pension = individu('pensions_nettes', annee_2021) / 12
        min_vi = individu.famille('aspa', annee_2021, options = [ADD]) / 12

        # pas non-salarié, salarié, agent public :
        pas_autre = (individu('eligibilite_indemnite_inflation_non_salarie', period)
                    + individu('eligibilite_indemnite_inflation_salarie_prive', period)
                    + individu('eligibilite_indemnite_inflation_public', period)) == 0

        return or_(and_(eligibilite_retraite, pension <= 2000),
                   and_(eligibilite_aspa, min_vi <= 2000)) * pas_autre


# 5 : Min Soc, Prest Soc
class eligibilite_indemnite_inflation_prest_soc(Variable):
    entity = Individu
    value_type = bool
    reference = 'https://www.legifrance.gouv.fr/jorf/id/JORFTEXT000044471405'
    label = "Eligibilité à l'indemnité inflation en tant que bénéficiaire des prestations sociales"
    definition_period = YEAR

    def formula(individu, period, parameters):

        oct_2021 = periods.period('2021-10')

        # pension d'invalidité <= 2000 par mois
        eligibilite_pension_invalidite = (individu('pensions_invalidite', oct_2021) <= 2000) * (individu('pensions_invalidite', oct_2021) > 0)

        # allocataire de l'AAH; RSA; ASI; PreParE; sans critère de montant
        eligibilite_allocations = (individu('aah', oct_2021)
                                   + individu.famille('rsa', oct_2021) * individu.has_role(Famille.PARENT)
                                   + individu('asi', oct_2021)
                                   + individu.famille('paje_prepare', oct_2021) * individu.has_role(Famille.PARENT))

        # pas non-salarié, salarié, agent public, retraité :
        pas_autre = (individu('eligibilite_indemnite_inflation_non_salarie', period)
                    + individu('eligibilite_indemnite_inflation_salarie_prive', period)
                    + individu('eligibilite_indemnite_inflation_public', period)
                    + individu('eligibilite_indemnite_inflation_retraite', period)) == 0

        return ((eligibilite_pension_invalidite + eligibilite_allocations) > 0) * pas_autre


# 6 : Jeunes
class eligibilite_indemnite_inflation_jeune(Variable):
    entity = Individu
    value_type = bool
    reference = 'https://www.legifrance.gouv.fr/jorf/id/JORFTEXT000044471405'
    label = "Eligibilité à l'indemnité inflation en tant que jeune"
    definition_period = YEAR

    def formula(individu, period, parameters):

        oct_2021 = periods.period('2021-10')

        # au moins 16 ans
        eligibilite_age = individu('age', oct_2021) >= 16

        # étudiant boursier
        eligibilite_etudiant_boursier = individu('bourse_criteres_sociaux', oct_2021) > 0

        # étudiant non-boursier & AL
        eligibilite_etudiant_nb_al = and_(and_(individu('etudiant', oct_2021), individu('bourse_criteres_sociaux', oct_2021) == 0),
                                          and_(individu.famille('aide_logement', oct_2021) > 0, individu.has_role(Famille.DEMANDEUR)))

        # apprenti ou contrat de professionnalisation
        eligibilite_apprenti = or_(and_(individu('apprenti', oct_2021), individu('remuneration_apprenti', oct_2021) <= 2000),
                                   and_(individu('professionnalisation', oct_2021), individu('professionnalisation', oct_2021) <= 2000))

        # stagiaire formation prof.
        eligibilite_stage_prof = and_(individu('revenus_stage_formation_pro', oct_2021) > 0,
        individu('revenus_stage_formation_pro', oct_2021) < 2000)

        # jeune en rechere d'emploi, pqrcours contractualisé d'accompagnement, garantie jeunes
        eligibilite_emploi_gj = individu('garantie_jeunes', oct_2021) > 0

        # pas non-salarié, salarié, agent public, retraité, MinSoc/PrestSoc :
        pas_autre = (individu('eligibilite_indemnite_inflation_non_salarie', period)
                    + individu('eligibilite_indemnite_inflation_salarie_prive', period)
                    + individu('eligibilite_indemnite_inflation_public', period)
                    + individu('eligibilite_indemnite_inflation_retraite', period)
                    + individu('eligibilite_indemnite_inflation_prest_soc', period)) == 0

        return eligibilite_age * ((eligibilite_etudiant_boursier
                                   + eligibilite_etudiant_nb_al
                                   + eligibilite_apprenti
                                   + eligibilite_stage_prof
                                   + eligibilite_emploi_gj) > 0) * pas_autre


# 7 : Demandeur d'Emploi
class eligibilite_indemnite_inflation_demandeur_emploi(Variable):
    entity = Individu
    value_type = bool
    reference = 'https://www.legifrance.gouv.fr/jorf/id/JORFTEXT000044471405'
    label = "Eligibilité à l'indemnité inflation en tant que demandeur d'emploi"
    definition_period = YEAR

    def formula(individu, period, parameters):

        oct_2021 = periods.period('2021-10')

        # chômeur en octobre 2021
        eligibilite_chomeur = (individu('activite', oct_2021) == TypesActivite.chomeur)

        allocation = individu('chomage_net', oct_2021)

        # pas non-salarié, salarié, agent public, retraité, MinSoc/PrestSoc, jeune :
        pas_autre = (individu('eligibilite_indemnite_inflation_non_salarie', period)
                    + individu('eligibilite_indemnite_inflation_salarie_prive', period)
                    + individu('eligibilite_indemnite_inflation_public', period)
                    + individu('eligibilite_indemnite_inflation_retraite', period)
                    + individu('eligibilite_indemnite_inflation_prest_soc', period)
                    + individu('eligibilite_indemnite_inflation_jeune', period)) == 0

        return eligibilite_chomeur * (allocation <= 2000) * pas_autre


# L'éligibilité finale de l'indemnité inflation
###################################################


class eligibilite_indemnite_inflation(Variable):
    entity = Individu
    value_type = bool
    reference = 'https://www.legifrance.gouv.fr/jorf/id/JORFTEXT000044471405'
    label = "Eligibilité à l'indemnité inflation"
    definition_period = YEAR

    def formula(individu, period, parameters):

        eligible_salarie_prive = individu('eligibilite_indemnite_inflation_salarie_prive', period)
        eligible_non_salarie = individu('eligibilite_indemnite_inflation_non_salarie', period)
        eligible_public = individu('eligibilite_indemnite_inflation_public', period)
        eligible_demandeur_emploi = individu('eligibilite_indemnite_inflation_demandeur_emploi', period)
        eligible_retraite = individu('eligibilite_indemnite_inflation_retraite', period)
        eligible_prest_soc = individu('eligibilite_indemnite_inflation_prest_soc', period)
        eligible_jeune = individu('eligibilite_indemnite_inflation_jeune', period)

        return (eligible_salarie_prive + eligible_non_salarie + eligible_public + eligible_demandeur_emploi + eligible_retraite + eligible_prest_soc + eligible_jeune) > 0


# L'aide finale de l'indemnité inflation
############################################

class indemnite_inflation(Variable):
    entity = Individu
    value_type = float
    label = 'Aide exceptionnelle de 100 euros pour les individus gagnant € 2000 ou moins'
    reference = 'https://www.legifrance.gouv.fr/jorf/id/JORFTEXT000044471405'
    definition_period = YEAR
    set_input = set_input_divide_by_period
    end = '2021-12-31'

    def formula_2021_01_01(individu, period, parameters):
        montant_indemnite = parameters(period).prestations_sociales.solidarite_insertion.autre_solidarite.indemnite_inflation
        eligibilite_indemnite_inflation = individu('eligibilite_indemnite_inflation', period.this_year)

        return montant_indemnite * (eligibilite_indemnite_inflation > 0)



from openfisca_france.model.base import *


class pensions_alimentaires_percues(Variable):
    cerfa_field = {
        0: '1AO',
        1: '1BO',
        2: '1CO',
        3: '1DO',
        4: '1EO',
        }
    value_type = float
    unit = 'currency'
    entity = Individu
    label = 'Pensions alimentaires perçues'
    definition_period = MONTH
    set_input = set_input_divide_by_period
    calculate_output = calculate_output_add


class pensions_alimentaires_percues_decl(Variable):
    value_type = bool
    default_value = True
    entity = Individu
    label = 'Pension déclarée'
    definition_period = YEAR


class pensions_alimentaires_versees_individu(Variable):
    value_type = float
    entity = Individu
    label = 'Pensions alimentaires versées pour un individu'
    definition_period = MONTH
    set_input = set_input_divide_by_period


class gains_exceptionnels(Variable):
    value_type = float
    entity = Individu
    label = 'Gains exceptionnels'
    definition_period = MONTH
    set_input = set_input_divide_by_period


class allocation_securisation_professionnelle(Variable):
    value_type = float
    entity = Individu
    label = 'Allocation de sécurisation professionnelle'
    definition_period = MONTH
    set_input = set_input_divide_by_period


class prime_forfaitaire_mensuelle_reprise_activite(Variable):
    value_type = float
    entity = Individu
    label = "Prime forfaitaire mensuelle pour la reprise d'activité"
    definition_period = MONTH
    set_input = set_input_divide_by_period


class indemnites_volontariat(Variable):
    value_type = float
    entity = Individu
    label = 'Indemnités de volontariat'
    definition_period = MONTH
    set_input = set_input_divide_by_period


class dedommagement_victime_amiante(Variable):
    value_type = float
    entity = Individu
    label = "Dédommagement versé aux victimes de l'amiante"
    definition_period = MONTH
    set_input = set_input_divide_by_period


class prestation_compensatoire(Variable):
    value_type = float
    entity = Individu
    label = 'Prestation compensatoire'
    definition_period = MONTH
    set_input = set_input_divide_by_period


class pensions_invalidite(Variable):
    value_type = float
    entity = Individu
    label = "Pensions d'invalidité"
    # Cette case est apparue dans la déclaration 2014
    # Auparavant, les pensions d'invalidité étaient incluses dans la case 1AS
    cerfa_field = {
        0: '1AZ',
        1: '1BZ',
        2: '1CZ',
        3: '1DZ',
        }
    # start_date = date(2014, 1, 1)
    definition_period = MONTH
    set_input = set_input_divide_by_period


class bourse_enseignement_sup(Variable):
    value_type = float
    entity = Individu
    label = "Toute bourse de l'enseignement supérieur"
    definition_period = MONTH
    set_input = set_input_divide_by_period


# Avoir fiscaux et crédits d'impôt
# f2ab déjà disponible
class f8ta(Variable):
    cerfa_field = '8TA'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = "Retenue à la source en France ou impôt payé à l'étranger"
    definition_period = YEAR


class f8vl(Variable):
    cerfa_field = '8VL'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = "Impôt payé à l'étranger sur revenus de capitaux mobiliers et plus-values ouvrant droit à un crédit d'impôt"
    # start_date = date(2016, 1, 1)
    definition_period = YEAR


class f8vm(Variable):
    cerfa_field = {
        0: '8VM',
        1: '8WM',
        2: '8UM',
        }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = "Impôt payé à l'étranger sur revenus de capitaux mobiliers et plus-values ouvrant droit à un crédit d'impôt"
    # start_date = date(2016, 1, 1)
    definition_period = YEAR


class f8th(Variable):
    cerfa_field = '8TH'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Retenue à la source élus locaux'
    end = '2016-12-31'
    definition_period = YEAR


class f8td_2002_2005(Variable):
    cerfa_field = '8TD'
    value_type = int
    entity = FoyerFiscal
    label = 'Contribution exceptionnelle sur les hauts revenus'
    # start_date = date(2002, 1, 1)
    end = '2005-12-31'
    definition_period = YEAR


class f8td(Variable):
    cerfa_field = '8TD'
    value_type = bool
    entity = FoyerFiscal
    label = 'Revenus non imposables dépassent la moitié du RFR'
    # start_date = date(2011, 1, 1)
    end = '2014-12-31'
    definition_period = YEAR


class f8ti(Variable):
    cerfa_field = '8TK'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = "Revenus de l'étranger exonérés d'impôt"
    definition_period = YEAR


# Auto-entrepreneur : versements libératoires d’impôt sur le revenu
class f8uy(Variable):
    cerfa_field = '8UY'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Auto-entrepreneur : versements libératoires d’impôt sur le revenu dont le remboursement est demandé'
    # start_date = date(2009, 1, 1)
    definition_period = YEAR



from numpy import round, logical_or as or_

from openfisca_france.model.base import *


class cf_enfant_a_charge(Variable):
    value_type = bool
    entity = Individu
    label = 'Complément familial - Enfant considéré à charge'
    definition_period = MONTH

    def formula(individu, period, parameters):
        est_enfant_dans_famille = individu('est_enfant_dans_famille', period)
        autonomie_financiere = individu('autonomie_financiere', period)
        age = individu('age', period)

        cf = parameters(period).prestations_sociales.prestations_familiales.prestations_generales.cf

        condition_age = (age >= 0) * (age < cf.cf_cm.age_max)
        condition_situation = est_enfant_dans_famille * not_(autonomie_financiere)

        return condition_age * condition_situation


class cf_enfant_eligible(Variable):
    value_type = bool
    entity = Individu
    label = "Complément familial - Enfant pris en compte pour l'éligibilité"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(individu, period, parameters):
        cf_enfant_a_charge = individu('cf_enfant_a_charge', period)
        age = individu('age', period)
        rempli_obligation_scolaire = individu('rempli_obligation_scolaire', period)

        enfants = parameters(period).prestations_sociales.prestations_familiales.def_pac.enfants
        cf = parameters(period).prestations_sociales.prestations_familiales.prestations_generales.cf

        condition_enfant = (
            (age >= cf.cf_cm.age_min)
            * (age < enfants.age_intermediaire)
            * rempli_obligation_scolaire
            )

        condition_jeune = (age >= enfants.age_intermediaire) * (age < cf.cf_cm.age_max)

        return or_(condition_enfant, condition_jeune) * cf_enfant_a_charge


class cf_dom_enfant_eligible(Variable):
    value_type = bool
    entity = Individu
    label = "Complément familial (DOM) - Enfant pris en compte pour l'éligibilité"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(individu, period, parameters):
        cf_enfant_a_charge = individu('cf_enfant_a_charge', period)
        age = individu('age', period)
        rempli_obligation_scolaire = individu('rempli_obligation_scolaire', period)

        cf = parameters(period).prestations_sociales.prestations_familiales.prestations_generales.cf

        condition_age = (age >= cf.cf_cm_dom.age_minimal_dom) * (age < cf.cf_cm_dom.age_maximal_dom)
        condition_situation = cf_enfant_a_charge * rempli_obligation_scolaire

        return condition_age * condition_situation


class cf_dom_enfant_trop_jeune(Variable):
    value_type = bool
    entity = Individu
    label = 'Complément familial (DOM) - Enfant trop jeune pour ouvrir le droit'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(individu, period, parameters):
        est_enfant_dans_famille = individu('est_enfant_dans_famille', period)
        age = individu('age', period)

        cf = parameters(period).prestations_sociales.prestations_familiales.prestations_generales.cf

        condition_age = (age >= 0) * (age < cf.cf_cm.age_min)

        return condition_age * est_enfant_dans_famille


class cf_base_ressources_individu(Variable):
    value_type = float
    entity = Individu
    label = "Complément familial - Ressources de l'individu prises en compte"
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period):
        base_ressources = individu('prestations_familiales_base_ressources_individu', period)
        est_enfant_dans_famille = individu('est_enfant_dans_famille', period)
        cf_enfant_a_charge = individu('cf_enfant_a_charge', period)

        return or_(not_(est_enfant_dans_famille), cf_enfant_a_charge) * base_ressources


class cf_plafond(Variable):
    value_type = float
    entity = Famille
    label = "Plafond d'éligibilité au Complément Familial"
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(famille, period, parameters):
        cf = parameters(period).prestations_sociales.prestations_familiales.prestations_generales.cf
        ars = parameters(period).prestations_sociales.prestations_familiales.education_presence_parentale.ars

        eligibilite_base = famille('cf_eligibilite_base', period)
        eligibilite_dom = famille('cf_eligibilite_dom', period)
        isole = not_(famille('en_couple', period))
        biactivite = famille('biactivite', period)

        # Calcul du nombre d'enfants à charge au sens du CF
        cf_enfant_a_charge_i = famille.members('cf_enfant_a_charge', period)
        cf_nbenf = famille.sum(cf_enfant_a_charge_i)

        # Calcul du taux à appliquer au plafond de base pour la France métropolitaine
        taux_plafond_metropole = (
            1
            + cf.cf_plaf.majoration.deux_premiers_enf
            * min_(cf_nbenf, 2)
            + cf.cf_plaf.majoration.troisieme_enf_et_plus
            * max_(cf_nbenf - 2, 0)
            )

        # Majoration du plafond pour biactivité ou isolement (France métropolitaine)
        majoration_plafond = (isole | biactivite)

        # Calcul du plafond pour la France métropolitaine
        plafond_metropole = (
            cf.cf_plaf.plafond_ressources_0_enfant
            * taux_plafond_metropole
            + cf.cf_plaf.majoration.biactifs_isoles
            * majoration_plafond
            )

        # Calcul du taux à appliquer au plafond de base pour les DOM
        taux_plafond_dom = 1 + cf_nbenf * ars.ars_plaf.majoration_par_enf_supp

        # Calcul du plafond pour les DOM
        plafond_dom = ars.ars_plaf.plafond_ressources * taux_plafond_dom

        plafond = (
            eligibilite_base
            * plafond_metropole
            + eligibilite_dom
            * plafond_dom
            )

        return plafond


class cf_majore_plafond(Variable):
    value_type = float
    entity = Famille
    label = "Plafond d'éligibilité au Complément Familial majoré"
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula_2014_04_01(famille, period, parameters):
        plafond_base = famille('cf_plafond', period)
        cf = parameters(period).prestations_sociales.prestations_familiales.prestations_generales.cf
        return plafond_base * cf.cf_plaf.plafond_cf_majore


class cf_base_ressources(Variable):
    value_type = float
    entity = Famille
    label = "Ressources prises en compte pour l'éligibilité au complément familial"
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(famille, period):
        cf_base_ressources_individu_i = famille.members('cf_base_ressources_individu', period)
        ressources_i_total = famille.sum(cf_base_ressources_individu_i)
        ressources_communes = famille('prestations_familiales_base_ressources_communes', period)
        return ressources_i_total + ressources_communes


class cf_eligibilite_base(Variable):
    value_type = bool
    entity = Famille
    label = 'Éligibilité au complément familial sous condition de ressources et avant cumul'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(famille, period, parameters):
        residence_dom = famille.demandeur.menage('residence_dom', period)

        cf_enfant_eligible = famille.members('cf_enfant_eligible', period)
        cf_nbenf = famille.sum(cf_enfant_eligible)

        return not_(residence_dom) * (cf_nbenf >= 3)


class cf_eligibilite_dom(Variable):
    value_type = bool
    entity = Famille
    label = 'Éligibilité au complément familial pour les DOM sous condition de ressources et avant cumul'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(famille, period, parameters):
        residence_dom = famille.demandeur.menage('residence_dom', period)
        residence_mayotte = famille.demandeur.menage('residence_mayotte', period)

        cf_dom_enfant_eligible = famille.members('cf_dom_enfant_eligible', period)
        cf_nbenf = famille.sum(cf_dom_enfant_eligible)

        cf_dom_enfant_trop_jeune = famille.members('cf_dom_enfant_trop_jeune', period)
        cf_nbenf_trop_jeune = famille.sum(cf_dom_enfant_trop_jeune)

        condition_composition_famille = (cf_nbenf >= 1) * (cf_nbenf_trop_jeune == 0)
        condition_residence = residence_dom * not_(residence_mayotte)

        return condition_composition_famille * condition_residence


class cf_non_majore_avant_cumul(Variable):
    value_type = float
    entity = Famille
    label = 'Complément familial non majoré avant cumul'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(famille, period, parameters):
        eligibilite_base = famille('cf_eligibilite_base', period)
        eligibilite_dom = famille('cf_eligibilite_dom', period)
        ressources = famille('cf_base_ressources', period)
        plafond = famille('cf_plafond', period)

        bmaf = parameters(period).prestations_sociales.prestations_familiales.bmaf.bmaf
        cf = parameters(period).prestations_sociales.prestations_familiales.prestations_generales.cf

        eligibilite_sous_condition = or_(eligibilite_base, eligibilite_dom)

        # Montant
        montant = (
            bmaf * (
                cf.cf_cm.complement_familial.taux_cf_base * eligibilite_base
                + cf.cf_cm_dom.complement_familial_dom.taux_base_dom * eligibilite_dom
                )
            )

        # Complément familial
        eligibilite = eligibilite_sous_condition * (ressources <= plafond)

        # Complément familial différentiel
        plafond_diff = plafond + 12 * montant

        eligibilite_diff = (
            not_(eligibilite)
            * eligibilite_sous_condition
            * (ressources <= plafond_diff)
            )

        montant_diff = (plafond_diff - ressources) / 12

        return max_(eligibilite * montant, eligibilite_diff * montant_diff)


class cf_majore_avant_cumul(Variable):
    value_type = float
    entity = Famille
    label = 'Complément familial majoré avant cumul'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula_2014_04_01(famille, period, parameters):
        eligibilite_base = famille('cf_eligibilite_base', period)
        eligibilite_dom = famille('cf_eligibilite_dom', period)
        ressources = famille('cf_base_ressources', period)
        plafond_majore = famille('cf_majore_plafond', period)

        bmaf = parameters(period).prestations_sociales.prestations_familiales.bmaf.bmaf
        cf = parameters(period).prestations_sociales.prestations_familiales.prestations_generales.cf

        eligibilite_sous_condition = or_(eligibilite_base, eligibilite_dom)

        # Montant
        montant = (
            bmaf * (
                cf.cf_cm.complement_familial.taux_cf_majore * eligibilite_base
                + cf.cf_cm_dom.complement_familial_dom.taux_majore_dom * eligibilite_dom
                )
            )

        eligibilite = eligibilite_sous_condition * (ressources <= plafond_majore)

        return eligibilite * montant


class cf_montant(Variable):
    value_type = float
    entity = Famille
    label = "Montant du complément familial, avant prise en compte d'éventuels cumuls"
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(famille, period):
        cf_non_majore_avant_cumul = famille('cf_non_majore_avant_cumul', period)
        cf_majore_avant_cumul = famille('cf_majore_avant_cumul', period)

        return max_(cf_non_majore_avant_cumul, cf_majore_avant_cumul)


class cf(Variable):
    calculate_output = calculate_output_add
    value_type = float
    entity = Famille
    label = 'Complément familial'
    reference = 'http://vosdroits.service-public.fr/particuliers/F13214.xhtml'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(famille, period, parameters):
        '''
        Pour les règles de non-cumul du CF avec les autres prestations, voir notamment les art. L532-1 et L532-2 du CSS
        '''
        paje_base = famille('paje_base', period)
        paje_clca = famille('paje_clca', period)
        paje_prepare = famille('paje_prepare', period)
        apje_avant_cumul = famille('apje_avant_cumul', period)
        ape_avant_cumul = famille('ape_avant_cumul', period)
        cf_montant = famille('cf_montant', period)
        residence_mayotte = famille.demandeur.menage('residence_mayotte', period)

        cf_brut = (
            not_(paje_base)
            * not_(paje_clca)
            * not_(paje_prepare)
            * (apje_avant_cumul <= cf_montant)
            * (ape_avant_cumul <= cf_montant)
            * cf_montant
            )

        return not_(residence_mayotte) * round(cf_brut, 2)



from numpy import ceil, divide, where, zeros_like

from openfisca_france.model.base import Individu, Variable, MONTH, \
    set_input_divide_by_period, set_input_dispatch_by_period, TypesActivite, \
    round_, max_, min_


class complement_are_eligibilite(Variable):
    value_type = bool
    entity = Individu
    label = "L'individu est éligible au Complément ARE car il perçoit un salaire"
    definition_period = MONTH
    reference = [
        'https://www.legifrance.gouv.fr/jorf/id/JORFTEXT000044345334',
        'https://www.legifrance.gouv.fr/loda/id/JORFTEXT000041798325/'
        ]
    set_input = set_input_divide_by_period

    def formula(individu, period):
        activite = individu('activite', period)
        reprise_activite = (activite == TypesActivite.actif)

        # le complément ARE prend le pas sur l'ARE lorsqu'un salaire est perçu
        return reprise_activite * individu('salaire_de_base', period) > 0


class complement_are_plafond(Variable):
    value_type = float
    entity = Individu
    label = 'Plafond de ressources pour bénéficiaire du complément ARE'
    definition_period = MONTH
    set_input = set_input_divide_by_period
    reference = 'https://www.unedic.org/indemnisation/fiches-thematiques/cumul-allocation-salaire'

    def formula(individu, period, parameters):
        salaire_journalier_reference = individu('are_salaire_journalier_reference', period)

        # Le gain brut est l'appelation métier utilisée dans le calcul du complément ARE
        # et représente la notion de salaire de reprise d'emploi
        gain_brut = individu('salaire_de_base', period)
        plafond = salaire_journalier_reference * parameters(period).chomage.allocations_assurance_chomage.complement_are.coefficient_plafond_global

        return max_(0, (plafond - gain_brut))


class complement_are_brut(Variable):
    value_type = float
    entity = Individu
    label = "Complément ARE brut pour reprise d'activité"
    definition_period = MONTH
    set_input = set_input_divide_by_period
    reference = 'https://www.unedic.org/indemnisation/fiches-thematiques/cumul-allocation-salaire'

    def formula(individu, period):
        complement_are_eligibilite = individu('complement_are_eligibilite', period)
        allocation_journaliere = individu('allocation_retour_emploi_journaliere', period)
        nombre_jours_indemnises = individu('complement_are_nombre_jours_indemnises', period)

        return complement_are_eligibilite * round_(allocation_journaliere * nombre_jours_indemnises, 2)


class complement_are_net(Variable):
    value_type = float
    entity = Individu
    label = 'Montant mensuel du complément ARE net de déductions'
    definition_period = MONTH
    set_input = set_input_divide_by_period
    reference = 'https://www.unedic.org/indemnisation/fiches-thematiques/cumul-allocation-salaire'
    documentation = '''
    Le Complément ARE net perçu permet de conserver une partie de ses allocations chômage
    à la reprise d'activité si le salaire de l'emploi repris est inférieur
    au salaire ayant initialement débloqué les droits au chômage.
    '''

    def formula(individu, period):
        allocation_mensuelle_due_brute = individu('complement_are_brut', period)
        deductions_montant_mensuel = individu('complement_are_deductions', period)  # montant négatif

        return max_(0, round_(allocation_mensuelle_due_brute + deductions_montant_mensuel, 2))


class complement_are_salaire_retenu(Variable):
    value_type = float
    entity = Individu
    label = "Montant de salaire retenu pour le calcul du complément ARE après déduction d'un taux"
    definition_period = MONTH
    set_input = set_input_divide_by_period
    reference = 'https://www.unedic.org/indemnisation/fiches-thematiques/cumul-allocation-salaire'

    def formula(individu, period, parameters):
        # Le gain brut est l'appelation métier utilisée dans le calcul du complément ARE
        # et représente la notion de salaire de reprise d'emploi
        gain_brut = individu('salaire_de_base', period)

        return round_(gain_brut * parameters(period).chomage.allocations_assurance_chomage.complement_are.taux_remuneration_retenue, 1)


class complement_are_base(Variable):
    value_type = float
    entity = Individu
    label = "Base d'ARE restante après déduction du salaire de reprise d'activité"
    definition_period = MONTH
    set_input = set_input_divide_by_period
    reference = 'https://www.unedic.org/indemnisation/fiches-thematiques/cumul-allocation-salaire'

    def formula(individu, period):
        are_brute_mensuelle = individu('allocation_retour_emploi_montant', period)
        salaire_retenu = individu('complement_are_salaire_retenu', period)

        # une part des revenus mensuels bruts issus de l’activité reprise
        # sont déduits du montant total de l’ARE qui aurait été versé
        # en l’absence de reprise d’activité
        return max_(0, are_brute_mensuelle - salaire_retenu)


class complement_are_nombre_jours_restants(Variable):
    value_type = float
    entity = Individu
    label = "Nombre de jours restants avant épuisement des droits à l'assurance chômage"
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period):
        nombre_jours_restants_fin_droits = individu(
            'complement_are_nombre_jours_restants_fin_droits',
            period.last_month
            )

        return max_(0, nombre_jours_restants_fin_droits)


class complement_are_nombre_jours_restants_fin_droits(Variable):
    value_type = float
    entity = Individu
    label = "Nombre de jours d'ARE restants avant épuisement des droits à la fin de la période considérée"
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period):
        nombre_jours_restants = individu('complement_are_nombre_jours_restants', period)
        nombre_jours_indemnisables = individu('complement_are_nombre_jours_indemnisables', period)

        return max_(0, nombre_jours_restants - nombre_jours_indemnisables)


class complement_are_nombre_jours_indemnisables(Variable):
    value_type = float
    entity = Individu
    label = "Nombre de jours d'ARE indemnisables par le complement ARE"
    definition_period = MONTH
    set_input = set_input_divide_by_period
    reference = 'https://www.legifrance.gouv.fr/loda/id/JORFTEXT000038829574'

    def formula(individu, period):
        allocation_journaliere_taux_plein = individu('allocation_retour_emploi_journaliere_taux_plein', period)
        allocation_journaliere = individu('allocation_retour_emploi_journaliere', period)
        degressivite_are = individu('degressivite_are', period)
        complement_are_base = individu('complement_are_base', period)

        nombre_jours_indemnisables = where(
            degressivite_are,
            round_(divide(complement_are_base, allocation_journaliere_taux_plein, out=zeros_like(complement_are_base), where=allocation_journaliere_taux_plein > 0)),
            round_(divide(complement_are_base, allocation_journaliere, out=zeros_like(complement_are_base), where=allocation_journaliere_taux_plein > 0))
            )

        return max_(0, nombre_jours_indemnisables)


class complement_are_nombre_jours_indemnises(Variable):
    value_type = float
    entity = Individu
    label = "Nombre de jours d'ARE indemnisés par le Complément ARE"
    definition_period = MONTH
    set_input = set_input_divide_by_period
    reference = 'https://www.legifrance.gouv.fr/loda/id/JORFTEXT000038829574'

    def formula(individu, period):
        nombre_jours_indemnisables = individu('complement_are_nombre_jours_indemnisables', period)
        nombre_jours_restants = individu('complement_are_nombre_jours_restants', period)

        return where(
            nombre_jours_indemnisables > nombre_jours_restants,
            nombre_jours_restants,
            nombre_jours_indemnisables
            )


class complement_are_deductions(Variable):
    value_type = float
    entity = Individu
    label = 'Charges mensuelles déductives du complément ARE'
    definition_period = MONTH
    set_input = set_input_divide_by_period
    reference = [
        'https://www.unedic.org/indemnisation/fiches-thematiques/cumul-allocation-salaire',
        'https://www.unedic.org/indemnisation/fiches-thematiques/retenues-sociales-sur-les-allocations'
        ]

    def formula(individu, period):
        cotisation_retraite_complementaire = individu('complement_are_cotisation_retraite_complementaire', period)
        csg = individu('complement_are_csg', period)
        crds = individu('complement_are_crds', period)

        return round_(cotisation_retraite_complementaire + csg + crds, 2)


class complement_are_cotisation_retraite_complementaire(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisation de retraite complémentaire mensuelle sur le Complément ARE'
    definition_period = MONTH
    set_input = set_input_divide_by_period
    reference = [
        'https://www.unedic.org/indemnisation/fiches-thematiques/cumul-allocation-salaire',
        'https://www.unedic.org/indemnisation/fiches-thematiques/retenues-sociales-sur-les-allocations'
        ]

    def formula(individu, period):
        cotisation_retraite_complementaire_journaliere = individu('chomage_cotisation_retraite_complementaire_journaliere', period)
        nombre_jours_indemnisables = individu('complement_are_nombre_jours_indemnisables', period)

        return cotisation_retraite_complementaire_journaliere * nombre_jours_indemnisables


class complement_are_csg_journaliere(Variable):
    value_type = float
    entity = Individu
    label = 'Contribution sociale généralisée (CSG) journalière sur le Complément ARE'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period
    reference = [
        'https://www.unedic.org/indemnisation/fiches-thematiques/retenues-sociales-sur-les-allocations',
        'https://www.unedic.org/sites/default/files/circulaires/PRE-CIRC-Circulaire_n_2021-13_du_19_octobre_2021.pdf'  # seuil d'exonération
        ]

    def formula(individu, period, parameters):
        allocation_journaliere_brute_are = individu('assiette_csg_crds_chomage_journaliere', period)  # CRC déduite

        parametres_prelevements_sociaux = parameters(period).prelevements_sociaux
        parametres_csg_chomage = parametres_prelevements_sociaux.contributions_sociales.csg.remplacement.allocations_chomage

        # abattement d'assiette pour frais sous 4 plafonds _mensuels_ de la sécurité sociale (PSS)
        max_assiette_mensuelle_eligible_abattement = 4 * parametres_prelevements_sociaux.pss.plafond_securite_sociale_mensuel
        complement_are_nombre_jours_indemnisables = individu('complement_are_nombre_jours_indemnisables', period)
        assiette_mensuelle_csg = allocation_journaliere_brute_are * complement_are_nombre_jours_indemnisables

        assiette_mensuelle_csg_abattable = min_(assiette_mensuelle_csg, max_assiette_mensuelle_eligible_abattement)
        assiette_mensuelle_csg_non_abattue = max_(assiette_mensuelle_csg - max_assiette_mensuelle_eligible_abattement, 0)

        abattement_assiette_csg = parametres_csg_chomage.imposable.abattement.rates[0]
        assiette_journaliere_csg = (
            (assiette_mensuelle_csg_abattable * (1 - abattement_assiette_csg)) + assiette_mensuelle_csg_non_abattue
            ) / complement_are_nombre_jours_indemnisables

        # taux plein par défaut : au demandeur d'emploi de suivre une démarche
        # pour la prise en compte du RFR déterminant les taux réduits
        taux_global_csg_chomage = parametres_csg_chomage.taux_global
        csg_theorique = assiette_journaliere_csg * taux_global_csg_chomage

        # la CSG ne doit pas faire baisser le montant net de l'allocation en-dessous du smic brut
        smic_horaire_brut = parameters(period).marche_travail.salaire_minimum.smic.smic_b_horaire
        seuil_journalier_exoneration = ceil(smic_horaire_brut * 35 / 7)
        allocation_csg_deduite = allocation_journaliere_brute_are - csg_theorique

        csg_prelevee = round_(
            where(
                allocation_csg_deduite > seuil_journalier_exoneration,
                -1 * csg_theorique,
                0),
            2)

        return csg_prelevee


class complement_are_csg(Variable):
    value_type = float
    entity = Individu
    label = 'Contribution sociale généralisée (CSG) mensuelle sur Complément ARE'
    definition_period = MONTH
    set_input = set_input_divide_by_period
    reference = 'https://www.unedic.org/indemnisation/fiches-thematiques/retenues-sociales-sur-les-allocations'

    def formula(individu, period):
        csg_journaliere = individu('complement_are_csg_journaliere', period)
        nombre_jours_indemnisables = individu('complement_are_nombre_jours_indemnisables', period)

        return csg_journaliere * nombre_jours_indemnisables


class complement_are_crds_journaliere(Variable):
    value_type = float
    entity = Individu
    label = 'Contribution au Remboursement de la Dette Sociale (CRDS) journalière sur Complément ARE'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period
    reference = [
        'https://www.unedic.org/indemnisation/fiches-thematiques/retenues-sociales-sur-les-allocations',
        'https://www.unedic.org/sites/default/files/circulaires/PRE-CIRC-Circulaire_n_2021-13_du_19_octobre_2021.pdf'  # seuil d'exonération
        ]

    def formula(individu, period, parameters):
        allocation_journaliere_brute_are = individu('assiette_csg_crds_chomage_journaliere', period)  # CRC déduite

        parametres_prelevements_sociaux = parameters(period).prelevements_sociaux

        # abattement d'assiette pour frais sous 4 plafonds _mensuels_ de la sécurité sociale identique CRDS activité
        complement_are_nombre_jours_indemnisables = individu('complement_are_nombre_jours_indemnisables', period)
        assiette_mensuelle_crds = allocation_journaliere_brute_are * complement_are_nombre_jours_indemnisables

        abattement_assiette_mensuelle_crds = (
            parametres_prelevements_sociaux.contributions_sociales.crds.activite.abattement.calc(
                assiette_mensuelle_crds,
                factor = parametres_prelevements_sociaux.pss.plafond_securite_sociale_mensuel
                )
            )
        assiette_journaliere_crds = (assiette_mensuelle_crds - abattement_assiette_mensuelle_crds) / complement_are_nombre_jours_indemnisables

        # taux global par défaut : au demandeur d'emploi de suivre une démarche
        # pour la prise en compte du RFR (exonération potentielle de CRDS)
        taux_global_crds_chomage = parametres_prelevements_sociaux.contributions_sociales.crds.taux_global
        crds_theorique = assiette_journaliere_crds * taux_global_crds_chomage

        # après la CSG, la CRDS ne doit pas faire baisser le montant net de l'allocation en-dessous du smic brut
        smic_horaire_brut = parameters(period).marche_travail.salaire_minimum.smic.smic_b_horaire
        seuil_journalier_exoneration = ceil(smic_horaire_brut * 35 / 7)

        complement_are_csg_journaliere = individu('complement_are_csg_journaliere', period)
        allocation_csg_crds_deduites = allocation_journaliere_brute_are - complement_are_csg_journaliere - crds_theorique

        crds_prelevee = round_(
            where(
                allocation_csg_crds_deduites > seuil_journalier_exoneration,
                -1 * crds_theorique,
                0),
            2)

        return crds_prelevee


class complement_are_crds(Variable):
    value_type = float
    entity = Individu
    label = 'Contribution au Remboursement de la Dette Sociale (CRDS) mensuelle sur Complément ARE'
    definition_period = MONTH
    set_input = set_input_divide_by_period
    reference = [
        'https://www.unedic.org/indemnisation/fiches-thematiques/cumul-allocation-salaire',
        'https://www.unedic.org/indemnisation/fiches-thematiques/retenues-sociales-sur-les-allocations'
        ]

    def formula(individu, period):
        crds_journaliere = individu('complement_are_crds_journaliere', period)
        nombre_jours_indemnisables = individu('complement_are_nombre_jours_indemnisables', period)

        return crds_journaliere * nombre_jours_indemnisables



from openfisca_france.model.base import Variable, Individu, MONTH, set_input_dispatch_by_period, set_input_divide_by_period
from openfisca_france.model.caracteristiques_socio_demographiques.logement import TypesLieuResidence


class pret_formation_permis_eligibilite(Variable):
    value_type = bool
    label = 'Éligibilité au prêt de formation au permis de conduire à 1 euro par jour'
    entity = Individu
    definition_period = MONTH
    reference = [
        'https://www.legifrance.gouv.fr/loda/id/LEGITEXT000006052491/',
        'https://www.securite-routiere.gouv.fr/passer-son-permis-de-conduire/financement-du-permis-de-conduire/permis-1-eu-par-jour'
        ]
    documentation = '''
    Le prêt « permis à un euro par jour » est exclusivement destiné au financement
    d’une formation initiale ou, dans le cas d’un échec à l’épreuve pratique
    de l’examen du permis de conduire, d’une formation complémentaire.
    Conditions non modélisées :
    Il ne peut être attribué qu’une seule fois à un même bénéficiaire et pour une même catégorie de permis.
    Ces formations doivent viser l’obtention du permis de conduire soit de la catégorie A1,
    soit de la catégorie A2, soit de la catégorie B.
    '''

    def formula_2005_09_30(individu, period, parameters):
        age = individu('age', period)
        criteres_age = parameters(period).prestations_sociales.transport.pret_formation_permis.age
        return (criteres_age.minimum <= age) * (age <= criteres_age.maximum)


class aide_financement_permis_apprenti(Variable):
    value_type = float
    label = "Montant de l'aide au financement du permis de conduire pour les apprentis"
    entity = Individu
    definition_period = MONTH
    set_input = set_input_divide_by_period
    reference = [
        "Décret n° 2019-1 du 3 janvier 2019 relatif à l'aide au financement du permis de conduire pour les apprentis",
        'https://www.legifrance.gouv.fr/eli/decret/2019/1/3/MTRD1835610D/jo/article_1'
        ]

    def formula(individu, period, parameters):
        montant = parameters(period).prestations_sociales.transport.aide_financement_permis_apprenti.montant
        eligibilite = individu('aide_financement_permis_apprenti_eligibilite', period)

        return montant * eligibilite


class aide_financement_permis_apprenti_eligibilite(Variable):
    value_type = bool
    label = "Éligibilité à l'aide au financement du permis de conduire pour les apprentis"
    entity = Individu
    definition_period = MONTH
    set_input = set_input_dispatch_by_period
    reference = [
        "Décret n° 2019-1 du 3 janvier 2019 relatif à l'aide au financement du permis de conduire pour les apprentis",
        'https://www.legifrance.gouv.fr/eli/decret/2019/1/3/MTRD1835610D/jo/article_1'
        ]

    def formula(individu, period, parameters):
        age_minimal = parameters(period).prestations_sociales.transport.aide_financement_permis_apprenti.age_min
        age = individu('age', period)
        apprenti = individu('apprenti', period)

        return (age_minimal <= age) * apprenti


class carte_sncf_eleve_apprenti_eligibilite(Variable):
    value_type = bool
    label = 'Éligibilité à la carte SNCF pour les élèves et les apprentis'
    entity = Individu
    definition_period = MONTH
    reference = [
        'https://www.sncf.com/fr/offres-voyageurs/cartes-tarifs-grandes-lignes/eleves-apprentis'
        ]

    def formula(individu, period, parameters):
        age_apprenti = parameters(period).prestations_sociales.transport.carte_sncf_eleve_apprenti.age_apprenti
        age_etudiant = parameters(period).prestations_sociales.transport.carte_sncf_eleve_apprenti.age_etudiant
        age = individu('age', period)

        eligibilite_apprenti = (age_apprenti > age) * individu('apprenti', period)
        eligibilite_etudiant = (age_etudiant > age) * individu('etudiant', period)

        residence_metropole = individu.menage('residence', period) == TypesLieuResidence.metropole

        return (eligibilite_apprenti + eligibilite_etudiant) * residence_metropole



import logging

from openfisca_france.model.base import *
from openfisca_france.model.prelevements_obligatoires.prelevements_sociaux.contributions_sociales.base import montant_csg_crds


log = logging.getLogger(__name__)


# TODO: prise_en_charge_employeur_retraite_supplementaire à la CSG/CRDS et au forfait social

# Salariés


class assiette_csg_abattue(Variable):
    value_type = float
    label = 'Assiette CSG - CRDS'
    reference = 'https://www.legifrance.gouv.fr/codes/article_lc/LEGIARTI000042683657'
    entity = Individu
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period, parameters):
        '''
        Assiette CSG - CRDS
            - 01/07/2022 : Ajout de la PPV à partir du 1er août 2022
        '''
        primes_salaires_non_exonerees = individu('primes_salaires_non_exonerees', period)
        prime_partage_valeur_exoneree = individu('prime_partage_valeur_exoneree', period, options=[DIVIDE])

        salaire_de_base = individu('salaire_de_base', period)
        primes_fonction_publique = individu('primes_fonction_publique', period)
        # indemnites_journalieres_maladie = individu('indemnites_journalieres_maladie', period)
        # TODO: mettre à part ?
        indemnite_residence = individu('indemnite_residence', period)
        indemnite_compensatrice_csg = individu('indemnite_compensatrice_csg', period)
        supplement_familial_traitement = individu('supplement_familial_traitement', period)
        hsup = individu('hsup', period)
        remuneration_principale = individu('remuneration_principale', period)
        stage_gratification_reintegration = individu('stage_gratification_reintegration', period)
        indemnite_fin_contrat = individu('indemnite_fin_contrat', period)
        avantage_en_nature = individu('avantage_en_nature', period)

        return (
            + indemnite_fin_contrat
            + indemnite_residence
            + indemnite_compensatrice_csg
            + primes_fonction_publique
            + primes_salaires_non_exonerees
            + prime_partage_valeur_exoneree
            + remuneration_principale
            + salaire_de_base
            + stage_gratification_reintegration
            + supplement_familial_traitement
            + avantage_en_nature
            - hsup
            )


class assiette_csg_non_abattue(Variable):
    value_type = float
    label = 'Assiette CSG - CRDS'
    entity = Individu
    definition_period = MONTH
    set_input = set_input_divide_by_period

    '''
    L'exclusion des contributions employeur aux contrats de prévoyance obligatoire n'est pas facile à dater, car elle a fait l'objet de différentes
    jurisprudences de la Cour de cassation, dans un sens comme dans l'autre, et de façon assez ancienne.
    voir par exemple Cour de Cassation, Chambre civile 2, du 23 novembre 2006, 05-11.364 : normalement, à cette date et par cette jurisprudence, les primes d'assurances
    prévoyance versées dans le cadre d'une obligation légale et n'acquérant pas de droits complémentaires au salarié sont exonérés de CSG.
    Cependant on retient la date de 2018, car la lecture de l'article L136-2 du CSS, ainsi que celle du cinquième alinéa de l'article L242-1 du CSS, ne font plus
    explicitement mention des contrats de prévoyance à cette date.
    Auparavant, et pour rester cohérent avec certains fichiers de test (notamment tests "fiches de paie"), on continue à inclure la prévoyance obligatoire
    (en particulier celle des cadres, qui est une obligation légale générale au-delà des conventions collectives) dans l'assiette de CSG.
    '''

    def formula_2018_01_01(individu, period, parameters):
        complementaire_sante_employeur = individu('complementaire_sante_employeur', period, options = [ADD])
        prevoyance_complementaire_employeur = individu('prevoyance_complementaire_employeur', period, options = [ADD])
        return (
            prevoyance_complementaire_employeur
            - complementaire_sante_employeur
            )

    def formula(individu, period, parameters):
        complementaire_sante_employeur = individu('complementaire_sante_employeur', period, options = [ADD])
        prevoyance_complementaire_employeur = individu('prevoyance_complementaire_employeur', period, options = [ADD])
        prevoyance_obligatoire_cadre = individu('prevoyance_obligatoire_cadre', period, options = [ADD])

        # TODO + indemnites_journalieres_maladie,
        return (
            - prevoyance_obligatoire_cadre
            + prevoyance_complementaire_employeur
            - complementaire_sante_employeur
            )


class csg_deductible_salaire(Variable):
    calculate_output = calculate_output_add
    value_type = float
    label = 'CSG déductible sur les salaires'
    reference = 'https://www.legifrance.gouv.fr/codes/section_lc/LEGITEXT000006073189/LEGISCTA000006173055/#LEGIARTI000042340733'
    entity = Individu
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period, parameters):
        assiette_csg_abattue = individu('assiette_csg_abattue', period)
        assiette_csg_non_abattue = individu('assiette_csg_non_abattue', period)
        plafond_securite_sociale = individu('plafond_securite_sociale', period)

        csg = parameters(period).prelevements_sociaux.contributions_sociales.csg
        montant_csg = montant_csg_crds(
            base_avec_abattement = assiette_csg_abattue,
            base_sans_abattement = assiette_csg_non_abattue,
            law_node = csg.activite.deductible,
            plafond_securite_sociale = plafond_securite_sociale,
            )
        return montant_csg


class csg_imposable_salaire(Variable):
    calculate_output = calculate_output_add
    value_type = float
    label = 'CSG imposables sur les salaires'
    entity = Individu
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period, parameters):
        assiette_csg_abattue = individu('assiette_csg_abattue', period)
        assiette_csg_non_abattue = individu('assiette_csg_non_abattue', period)
        plafond_securite_sociale = individu('plafond_securite_sociale', period)
        parameters = parameters(period)

        montant_csg = montant_csg_crds(
            base_avec_abattement = assiette_csg_abattue,
            base_sans_abattement = assiette_csg_non_abattue,
            law_node = parameters.prelevements_sociaux.contributions_sociales.csg.activite.imposable,
            plafond_securite_sociale = plafond_securite_sociale,
            )

        return montant_csg


class crds_salaire(Variable):
    calculate_output = calculate_output_add
    value_type = float
    label = 'CRDS sur les salaires'
    entity = Individu
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period, parameters):
        assiette_csg_abattue = individu('assiette_csg_abattue', period)
        assiette_csg_non_abattue = individu('assiette_csg_non_abattue', period)
        plafond_securite_sociale = individu('plafond_securite_sociale', period)

        law = parameters(period)

        montant_crds = montant_csg_crds(
            law_node = law.prelevements_sociaux.contributions_sociales.crds.activite,
            base_avec_abattement = assiette_csg_abattue,
            base_sans_abattement = assiette_csg_non_abattue,
            plafond_securite_sociale = plafond_securite_sociale,
            )

        return montant_crds


class forfait_social(Variable):
    value_type = float
    entity = Individu
    label = 'Forfait social'
    definition_period = MONTH
    calculate_output = calculate_output_add
    set_input = set_input_divide_by_period

    # les contributions destinées au financement des prestations de prévoyance complémentaire versées
    # au bénéfice de leurs salariés, anciens salariés et de leurs ayants droit (entreprises à partir de 10 salariés),
    # la réserve spéciale de participation dans les sociétés coopératives ouvrières de production (Scop).

    def formula_2009_01_01(individu, period, parameters):
        prise_en_charge_employeur_retraite_complementaire = individu('prise_en_charge_employeur_retraite_complementaire', period, options = [ADD])
        parametres = parameters(period).prelevements_sociaux.contributions_assises_specifiquement_accessoires_salaire.forfait_social
        taux_plein = parametres.taux_plein
        assiette_taux_plein = prise_en_charge_employeur_retraite_complementaire  # TODO: compléter l'assiette

        return - assiette_taux_plein * taux_plein

    def formula_2012_08_01(individu, period, parameters):
        prise_en_charge_employeur_retraite_complementaire = individu('prise_en_charge_employeur_retraite_complementaire', period, options = [ADD])
        parametres = parameters(period).prelevements_sociaux.contributions_assises_specifiquement_accessoires_salaire.forfait_social
        taux_plein = parametres.taux_plein
        assiette_taux_plein = prise_en_charge_employeur_retraite_complementaire  # TODO: compléter l'assiette

        # Les cotisations de prévoyance complémentaire qui rentrent en compte dans l'assiette du taux réduit
        # ne concernent que les entreprises de 10 ou 11 employés et plus
        # https://www.urssaf.fr/portail/home/employeur/calculer-les-cotisations/les-taux-de-cotisations/le-forfait-social/le-forfait-social-au-taux-de-8.html
        seuil_effectif_taux_reduit = parametres.seuil_effectif_prevoyance_complementaire
        prevoyance_complementaire_employeur = individu('prevoyance_complementaire_employeur', period, options = [ADD])
        prevoyance_obligatoire_cadre = individu('prevoyance_obligatoire_cadre', period, options = [ADD])
        effectif_entreprise = individu('effectif_entreprise', period)
        complementaire_sante_employeur = individu('complementaire_sante_employeur', period, options = [ADD])
        taux_reduit = parametres.taux_reduit_1  # TODO taux_reduit_2 in 2016
        assiette_taux_reduit = (
            - prevoyance_obligatoire_cadre + prevoyance_complementaire_employeur
            - complementaire_sante_employeur
            ) * (effectif_entreprise >= seuil_effectif_taux_reduit)

        return - (
            assiette_taux_plein * taux_plein + assiette_taux_reduit * taux_reduit
            )

    def formula_2022_07_01(individu, period, parameters):
        # Seule la PPV pérenne est sousmise au forfait social, et cela intégralement
        prime_partage_valeur = individu('prime_partage_valeur', period, options=[DIVIDE])

        prise_en_charge_employeur_retraite_complementaire = individu('prise_en_charge_employeur_retraite_complementaire', period, options=[ADD])
        effectif_entreprise = individu('effectif_entreprise', period)
        parametres = parameters(period).prelevements_sociaux.contributions_assises_specifiquement_accessoires_salaire.forfait_social
        taux_plein = parametres.taux_plein
        # TODO : faire ça propre ! Il faut externaliser le paramètre.
        prime_partage_valeur_a_integrer = prime_partage_valeur * (
            effectif_entreprise >= 250
            )
        assiette_taux_plein = (
            prise_en_charge_employeur_retraite_complementaire  # TODO: compléter l'assiette
            + prime_partage_valeur_a_integrer
            )

        # Les cotisations de prévoyance complémentaire qui rentrent en compte dans l'assiette du taux réduit
        # ne concernent que les entreprises de 10 ou 11 employés et plus
        # https://www.urssaf.fr/portail/home/employeur/calculer-les-cotisations/les-taux-de-cotisations/le-forfait-social/le-forfait-social-au-taux-de-8.html
        seuil_effectif_taux_reduit = parametres.seuil_effectif_prevoyance_complementaire
        prevoyance_complementaire_employeur = individu('prevoyance_complementaire_employeur', period, options=[ADD])
        prevoyance_obligatoire_cadre = individu('prevoyance_obligatoire_cadre', period, options = [ADD])

        complementaire_sante_employeur = individu('complementaire_sante_employeur', period, options=[ADD])
        taux_reduit = parametres.taux_reduit_1  # TODO taux_reduit_2 in 2016

        assiette_taux_reduit = (
            - prevoyance_obligatoire_cadre + prevoyance_complementaire_employeur
            - complementaire_sante_employeur
            ) * (effectif_entreprise >= seuil_effectif_taux_reduit)

        return -(assiette_taux_plein * taux_plein + assiette_taux_reduit * taux_reduit)


class salaire_imposable(Variable):
    value_type = float
    unit = 'currency'
    cerfa_field = {  # (f1aj, f1bj, f1cj, f1dj, f1ej)
        0: '1AJ',
        1: '1BJ',
        2: '1CJ',
        3: '1DJ',
        4: '1EJ',
        }
    entity = Individu
    label = 'Salaires imposables'
    reference = 'https://www.legifrance.gouv.fr/codes/article_lc/LEGIARTI000042683657'
    set_input = set_input_divide_by_period
    definition_period = MONTH

    def formula(individu, period):
        '''
        Salaires imposables
            - 01/07/2022 : Ajout PPV : prime_partage_valeur
        '''
        salaire_de_base = individu('salaire_de_base', period)
        primes_salaires_non_exonerees = individu('primes_salaires_non_exonerees', period)
        prime_partage_valeur_exoneree = individu('prime_partage_valeur_exoneree', period, options=[DIVIDE])

        primes_fonction_publique = individu('primes_fonction_publique', period)
        indemnite_residence = individu('indemnite_residence', period)
        indemnite_compensatrice_csg = individu('indemnite_compensatrice_csg', period)
        supplement_familial_traitement = individu('supplement_familial_traitement', period)
        csg_deductible_salaire = individu('csg_deductible_salaire', period)
        cotisations_salariales = individu('cotisations_salariales', period)
        remuneration_principale = individu('remuneration_principale', period)
        hsup = individu('hsup', period)
        indemnite_fin_contrat = individu('indemnite_fin_contrat', period)
        complementaire_sante_salarie = individu('complementaire_sante_salarie', period)

        return (
            salaire_de_base
            + primes_salaires_non_exonerees
            + prime_partage_valeur_exoneree
            + remuneration_principale
            + primes_fonction_publique
            + indemnite_residence
            + supplement_familial_traitement
            + csg_deductible_salaire
            + cotisations_salariales
            - hsup
            + indemnite_fin_contrat
            + complementaire_sante_salarie
            + indemnite_compensatrice_csg
            )


class salaire_net(Variable):
    value_type = float
    entity = Individu
    label = "Salaires nets d'après définition INSEE"
    set_input = set_input_divide_by_period
    definition_period = MONTH

    def formula(individu, period, parameters):
        '''
        Calcul du salaire net d'après définition INSEE
        net = net de csg et crds
        '''
        salaire_imposable = individu('salaire_imposable', period)
        crds_salaire = individu('crds_salaire', period)
        csg_imposable_salaire = individu('csg_imposable_salaire', period)

        return salaire_imposable + crds_salaire + csg_imposable_salaire

    def formula_2019_01_01(individu, period, parameters):
        '''
        Calcul du salaire net d'après définition INSEE
        net = net de csg et crds
        '''
        salaire_imposable = individu('salaire_imposable', period)
        crds_salaire = individu('crds_salaire', period)
        csg_imposable_salaire = individu('csg_imposable_salaire', period)
        prime_exceptionnelle_pouvoir_achat_exoneree = individu('prime_exceptionnelle_pouvoir_achat_exoneree', period, options = [DIVIDE])
        return salaire_imposable + crds_salaire + csg_imposable_salaire + prime_exceptionnelle_pouvoir_achat_exoneree

    def formula_2022_07_01(individu, period, parameters):
        '''
        Calcul du salaire net d'après définition INSEE
        net = net de csg et crds
        '''
        salaire_imposable = individu('salaire_imposable', period)
        crds_salaire = individu('crds_salaire', period)
        csg_imposable_salaire = individu('csg_imposable_salaire', period)
        prime_partage_valeur_exoneree_exceptionnelle = individu(
            'prime_partage_valeur_exoneree_exceptionnelle',
            period,
            options=[DIVIDE],
            )
        return (
            salaire_imposable  # La prime pérenne est dans le salaire_imposable
            + crds_salaire
            + csg_imposable_salaire
            + prime_partage_valeur_exoneree_exceptionnelle
            )


class tehr(Variable):
    value_type = float
    entity = Individu
    label = 'Taxe exceptionnelle sur les hautes rémunérations (TEHR)'
    reference = [
        'Article 15 de la loi 2013-1278',
        'https://www.legifrance.gouv.fr/loda/article_lc/LEGIARTI000028402680/'
        ]
    calculate_output = calculate_output_divide
    definition_period = YEAR
    end = '2015-01-01'

    def formula(individu, period, parameters):
        salaire_de_base = individu('salaire_de_base', period, options = [ADD])  # TODO: check base
        bareme_tehr = parameters(period).prelevements_sociaux.autres_taxes_participations_assises_salaires.tehr.tehr
        return -bareme_tehr.calc(salaire_de_base)


# Non salariés


class assiette_csg_crds_non_salarie(Variable):
    '''Assiette CSG des personnes non salariées'''
    value_type = float
    entity = Individu
    label = 'Assiette CSG des personnes non salariées'
    definition_period = YEAR

    def formula(individu, period):
        rpns_imposables = individu('rpns_imposables', period)
        categorie_non_salarie = individu('categorie_non_salarie', period)
        artisan = (categorie_non_salarie == TypesCategorieNonSalarie.artisan)
        commercant = (categorie_non_salarie == TypesCategorieNonSalarie.commercant)
        profession_liberale = (categorie_non_salarie == TypesCategorieNonSalarie.profession_liberale)
        famille_independant = individu('famille_independant', period)
        retraite_complementaire_artisan_commercant = individu('retraite_complementaire_artisan_commercant', period)
        maladie_maternite_artisan_commercant = individu('maladie_maternite_artisan_commercant', period)
        vieillesse_artisan_commercant = individu('vieillesse_artisan_commercant', period)
        maladie_maternite_profession_liberale = individu('maladie_maternite_profession_liberale', period)
        vieillesse_profession_liberale = individu('vieillesse_profession_liberale', period)
        retraite_complementaire_profession_liberale = individu('retraite_complementaire_profession_liberale', period)   # noqa F841

        assiette_cotisation = (
            (artisan + commercant + profession_liberale) * rpns_imposables
            - (  # cotisations are négative
                (artisan + commercant) * (
                    famille_independant
                    + vieillesse_artisan_commercant
                    + maladie_maternite_artisan_commercant
                    + retraite_complementaire_artisan_commercant
                    )
                + profession_liberale * (
                    famille_independant
                    + maladie_maternite_profession_liberale
                    + vieillesse_profession_liberale
                    )
                )
            )
        return assiette_cotisation


class csg_imposable_non_salarie(Variable):
    value_type = float
    entity = Individu
    label = 'Assiette CSG des personnes non salariées'
    definition_period = YEAR

    def formula(individu, period, parameters):
        assiette_csg_crds_non_salarie = individu('assiette_csg_crds_non_salarie', period)
        csg = parameters(period).prelevements_sociaux.contributions_sociales.csg.activite
        taux = csg.imposable.taux
        return - taux * assiette_csg_crds_non_salarie


class csg_deductible_non_salarie(Variable):
    value_type = float
    entity = Individu
    label = 'Assiette CSG des personnes non salariées'
    definition_period = YEAR

    def formula(individu, period, parameters):
        assiette_csg_crds_non_salarie = individu('assiette_csg_crds_non_salarie', period)
        csg = parameters(period).prelevements_sociaux.contributions_sociales.csg.activite
        taux = csg.deductible.taux
        return - taux * assiette_csg_crds_non_salarie


class crds_non_salarie(Variable):
    value_type = float
    entity = Individu
    label = 'Assiette CSG des personnes non salariées'
    definition_period = YEAR

    def formula(individu, period, parameters):
        assiette_csg_crds_non_salarie = individu('assiette_csg_crds_non_salarie', period)
        taux = parameters(period).prelevements_sociaux.contributions_sociales.crds.activite.taux
        return - taux * assiette_csg_crds_non_salarie


class revenus_non_salarie_nets(Variable):
    value_type = float
    entity = Individu
    label = 'Revenus du travail non salariaux nets'
    definition_period = YEAR

    def formula(individu, period):
        rpns_imposables = individu('rpns_imposables', period)
        csg_imposable_non_salarie = individu('csg_imposable_non_salarie', period)
        crds_non_salarie = individu('crds_non_salarie', period)
        microentreprise_i = individu.foyer_fiscal('microentreprise', period) * individu.has_role(FoyerFiscal.DECLARANT_PRINCIPAL)
        microentreprise = sum(microentreprise_i)
        return rpns_imposables + csg_imposable_non_salarie + crds_non_salarie + microentreprise



from numpy import datetime64, logical_or as or_

from openfisca_core.periods import Period

from openfisca_france.model.base import *


class autonomie_financiere(Variable):
    value_type = bool
    entity = Individu
    label = "Indicatrice d'autonomie financière vis-à-vis des prestations familiales"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period
    reference = [
        'https://www.legifrance.gouv.fr/affichCodeArticle.do?idArticle=LEGIARTI000006750602&cidTexte=LEGITEXT000006073189',
        'https://www.service-public.fr/particuliers/vosdroits/F16947'
        ]

    def formula(individu, period, parameters):
        # D'après service-public.fr, la condition de dépassement du salaire plafonds n'est pas évalué de la même manière suivant si l'enfant est étudiant ou salarié/apprenti/stagiaire.
        salaire_net_mensualise = individu('salaire_net', Period(('month', period.start, 6)).offset(-6), options = [ADD]) / 6

        _P = parameters(period)

        nbh_travaillees = 169
        smic_mensuel_brut = _P.marche_travail.salaire_minimum.smic.smic_b_horaire * nbh_travaillees

        return salaire_net_mensualise >= (_P.prestations_sociales.prestations_familiales.def_pac.revenu_plafond_pac_non_scolaire * smic_mensuel_brut)


class prestations_familiales_enfant_a_charge(Variable):
    value_type = bool
    entity = Individu
    label = 'Enfant considéré à charge au sens des prestations familiales'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(individu, period, parameters):
        est_enfant_dans_famille = individu('est_enfant_dans_famille', period)
        autonomie_financiere = individu('autonomie_financiere', period)
        age = individu('age', period)
        rempli_obligation_scolaire = individu('rempli_obligation_scolaire', period)

        pfam = parameters(period).prestations_sociales.prestations_familiales

        condition_enfant = (
            (age >= pfam.def_pac.enfants.age_minimal)
            * (age < pfam.def_pac.enfants.age_intermediaire)
            * rempli_obligation_scolaire
            )

        condition_jeune = (
            (age >= pfam.def_pac.enfants.age_intermediaire)
            * (age < pfam.def_pac.enfants.age_limite)
            * not_(autonomie_financiere)
            )

        return or_(condition_enfant, condition_jeune) * est_enfant_dans_famille


class prestations_familiales_base_ressources_individu(Variable):
    value_type = float
    is_period_size_independent = True
    entity = Individu
    label = 'Base ressource individuelle des prestations familiales'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period):
        annee_fiscale_n_2 = period.n_2

        traitements_salaires_pensions_rentes = individu('traitements_salaires_pensions_rentes', annee_fiscale_n_2)
        hsup = individu('hsup', annee_fiscale_n_2, options = [ADD])
        glo = individu('glo', annee_fiscale_n_2)
        plus_values = individu.foyer_fiscal('assiette_csg_plus_values', annee_fiscale_n_2) * individu.has_role(FoyerFiscal.DECLARANT_PRINCIPAL)
        rpns = individu('rpns_imposables', annee_fiscale_n_2)
        rpns_pvce = individu('rpns_pvce', annee_fiscale_n_2)
        rpns_exon = individu('rpns_exon', annee_fiscale_n_2)

        return traitements_salaires_pensions_rentes + hsup + glo + plus_values + rpns + rpns_pvce + rpns_exon


class biactivite(Variable):
    value_type = bool
    entity = Famille
    label = 'Indicatrice de biactivité'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(famille, period, parameters):
        '''
        Hypothèses/points à éclaircir :
           (1) A partir d'une certaine date sont apparemment pris en compte
               dans les "revenus professinnels" les indemnités journalières.
               Cf. circulaire interministérielle DSS/2B n°2011-447 du 01/12/2011
               Regarder davantage ce point
           (2) On n'a pas pris en compte les abbattements présents dans la
               variable abattement_salaires_pensions, car il s'agit d'une
               variable dépendant conjointement des salaires et des pensions
               (or, les pensions ne doivent pas être pris en compte ici, et
               cette variable ne peut pas être décomposée entre une part salaire et
               une part pensions). Mais cette variable n'existe que jusqu'à 2005
               inclus.
        '''
        annee_fiscale_n_2 = period.n_2

        bmaf = parameters(annee_fiscale_n_2).prestations_sociales.prestations_familiales.bmaf.bmaf
        seuil_rev = 12 * bmaf

        condition_ressource = (
            famille.members('rpns_imposables', annee_fiscale_n_2)
            + famille.members('revenu_assimile_salaire_apres_abattements', annee_fiscale_n_2)
            >= seuil_rev
            )
        deux_parents = famille.nb_persons(role = Famille.PARENT) == 2

        return deux_parents * famille.all(condition_ressource, role = Famille.PARENT)


class rev_coll(Variable):
    value_type = float
    entity = FoyerFiscal
    label = 'Revenus perçus par le foyer fiscal à prendre en compte dans la base ressource des prestations familiales'
    definition_period = YEAR

    def formula(foyer_fiscal, period):
        # Quand rev_coll est calculé sur une année glissante, rente_viagere_titre_onereux_net et pensions_alimentaires_versees sont calculés sur l'année légale correspondante.
        rente_viagere_titre_onereux_net = foyer_fiscal('rente_viagere_titre_onereux_net', period)
        pensions_alimentaires_versees = foyer_fiscal('pensions_alimentaires_versees', period)
        rev_cat_rvcm = foyer_fiscal('revenu_categoriel_capital', period)  # Supprimée en 2018
        revenus_capitaux_prelevement_liberatoire = foyer_fiscal('revenus_capitaux_prelevement_liberatoire', period, options = [ADD])  # Supprimée en 2018
        revenus_capitaux_prelevement_forfaitaire_unique_ir = foyer_fiscal('revenus_capitaux_prelevement_forfaitaire_unique_ir', period, options = [ADD])  # Existe à partir de 2018
        abat_spe = foyer_fiscal('abattements_speciaux_prestations_familiales', period)
        revenu_categoriel_foncier = foyer_fiscal('revenu_categoriel_foncier', period)
        f7ga = foyer_fiscal('f7ga', period)
        f7gb = foyer_fiscal('f7gb', period)
        f7gc = foyer_fiscal('f7gc', period)
        # est supprimée à partir de 2018
        rev_cat_pv = foyer_fiscal('revenu_categoriel_plus_values', period)
        plus_values_prelevement_forfaitaire_unique_ir = foyer_fiscal('plus_values_prelevement_forfaitaire_unique_ir', period)  # Apparait à partir de 2018

        # TODO: ajouter les revenus de l'étranger etr*0.9
        return (
            revenu_categoriel_foncier
            + pensions_alimentaires_versees  # négatif
            + rente_viagere_titre_onereux_net
            + rev_cat_rvcm
            + revenus_capitaux_prelevement_liberatoire
            + revenus_capitaux_prelevement_forfaitaire_unique_ir
            + rev_cat_pv
            + plus_values_prelevement_forfaitaire_unique_ir
            - abat_spe
            - f7ga
            - f7gb
            - f7gc
            )


class prestations_familiales_base_ressources_communes(Variable):
    value_type = float
    entity = Famille
    label = 'Ressources non individualisables prises en compte pour les prestations familiales'
    reference = [
        'Article D521-4 du Code de la sécurité sociale',
        'https://www.legifrance.gouv.fr/affichCodeArticle.do?idArticle=LEGIARTI000030678081&cidTexte=LEGITEXT000006073189&categorieLien=id'
        ]
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(famille, period):
        annee_fiscale_n_2 = period.n_2

        demandeur_declarant_principal = famille.demandeur.has_role(FoyerFiscal.DECLARANT_PRINCIPAL)
        conjoint_declarant_principal = famille.conjoint.has_role(FoyerFiscal.DECLARANT_PRINCIPAL)

        rev_coll = (
            famille.demandeur.foyer_fiscal('rev_coll', annee_fiscale_n_2)
            * demandeur_declarant_principal
            + famille.conjoint.foyer_fiscal('rev_coll', annee_fiscale_n_2)
            * conjoint_declarant_principal
            )

        return rev_coll


class prestations_familiales_base_ressources(Variable):
    value_type = float
    entity = Famille
    label = 'Base ressource des prestations familiales'
    reference = [
        'Article D521-4 du Code de la sécurité sociale',
        'https://www.legifrance.gouv.fr/affichCodeArticle.do?idArticle=LEGIARTI000030678081&cidTexte=LEGITEXT000006073189&categorieLien=id'
        ]
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(famille, period):
        base_ressources_i = famille.members('prestations_familiales_base_ressources_individu', period)
        enfant_i = famille.members('est_enfant_dans_famille', period)
        enfant_a_charge_i = famille.members('prestations_familiales_enfant_a_charge', period)
        ressources_i = (not_(enfant_i) + enfant_a_charge_i) * base_ressources_i
        base_ressources_i_total = famille.sum(ressources_i)

        ressources_communes = famille('prestations_familiales_base_ressources_communes', period)

        return base_ressources_i_total + ressources_communes


############################################################################
# Helper functions
############################################################################


def nb_enf(famille, period, age_min, age_max):
    '''
    Renvoie le nombre d'enfant au sens des allocations familiales dont l'âge est compris entre ag1 et ag2
    '''

    assert period.unit == 'month'
    assert period.size == 1

    age = famille.members('age', period)
    autonomie_financiere = famille.members('autonomie_financiere', period)

#        Les allocations sont dues à compter du mois civil qui suit la naissance
#        ag1==0 ou suivant les anniversaires ag1>0.
#        Un enfant est reconnu à charge pour le versement des prestations
#        jusqu'au mois précédant son age limite supérieur (ag2 + 1) mais
#        le versement à lieu en début de mois suivant
    condition = (
        (age >= age_min)
        * (age <= age_max)
        * not_(autonomie_financiere)
        )

    return famille.sum(condition, role = Famille.ENFANT)


class abattements_speciaux_prestations_familiales(Variable):
    value_type = float
    entity = FoyerFiscal
    label = 'Abattements spéciaux concernant les personnes agées ou invalides et les enfants à charge'
    reference = [
        'http://bofip.impots.gouv.fr/bofip/2036-PGP',
        'https://www.legifrance.gouv.fr/jorf/article_jo/JORFARTI000002185185'
        ]
    definition_period = YEAR

    def formula(foyer_fiscal, period, parameters):
        dateLimite = datetime64('1931-01-01')

        age_declarant = foyer_fiscal.declarant_principal('age', period.first_month)
        date_naissance_declarant = foyer_fiscal.declarant_principal('date_naissance', period.first_month)

        # Titulaire d'une pension pour une invalidité d'au moins 40 % ou d'une carte
        # d'invalidité d'au moins 80%
        declarant_invalide = foyer_fiscal('caseP', period.this_year)

        age_conjoint = foyer_fiscal.declarant_principal('age', period.first_month)
        date_naissance_conjoint = foyer_fiscal.conjoint('date_naissance', period.first_month)

        # Conjoint·e titulaire d'une pension ou d'une carte d'invalidité (vivant ou
        # décédé l'année de perception des revenus)
        conjoint_invalide = foyer_fiscal('caseF', period.this_year)

        # Revenu net global
        revenu_net_global = foyer_fiscal('rng', period)

        # Nombre d'enfants marié·e·s/pacse·é·s et d'enfants non mari·é·s charg·é·s de
        # famille
        nombre_enfants = foyer_fiscal('nbN', period)

        # Abattements pour revenu net imposable
        abattements = parameters(period).impot_revenu.calcul_revenus_imposables.abat_rni

        # Abattement pour personnes agées de + de 65 ans ou invalide
        abattement_age_ou_invalidite = abattements.personne_agee_ou_invalide

        # Abattement pour rattachement d'enfants mari·é·s
        abattement_enfant_marie = abattements.enfant_marie

        # Vecteur de foyers eligibles aux abattements spéciaux
        foyers_eligibles = (
            (((date_naissance_declarant < dateLimite) | declarant_invalide) & (age_declarant > 0))
            + (((date_naissance_conjoint < dateLimite) | conjoint_invalide) & (age_conjoint > 0))
            )

        # Vecteur de montants d'abattement pour personnes âges ou invalides
        abattement_special_personne_agee_invalide = (
            foyers_eligibles
            * (
                (
                    abattement_age_ou_invalidite.montant_1
                    * (revenu_net_global <= abattement_age_ou_invalidite.plafond_1)
                    )
                + (
                    abattement_age_ou_invalidite.montant_2
                    * (
                        (revenu_net_global > abattement_age_ou_invalidite.plafond_1)
                        & (revenu_net_global <= abattement_age_ou_invalidite.plafond_2)
                        )
                    )
                )
            )

        # Vecteur de montants d'abattement pour enfants à charge
        abattement_special_enfants_a_charge = (
            nombre_enfants
            * abattement_enfant_marie.montant
            )

        # Le montant total d'abattement ne peut pas être supérieur au revenu net global
        return min_(revenu_net_global, abattement_special_personne_agee_invalide + abattement_special_enfants_a_charge)



'''Test scenarios.'''


from openfisca_france.scenarios import init_single_entity

from .cache import tax_benefit_system

import pytest


def test_init_single_entity_parallel_axes():
    '''Test parallel axes scenario initialisation.'''
    year = 2019
    count = 3
    indexes = [0, 1, 2]
    salaire_de_base_axes = [
        dict(
            count = count,
            index = index,
            min = 0,
            max = 15000,
            name = 'salaire_de_base',
            period = year,
            )
        for index in indexes
        ]

    axes = [salaire_de_base_axes]

    scenario_kwargs = dict(
        parent1 = dict(age = 40),
        parent2 = dict(age = 40),
        enfants = [dict(age = 20)],
        axes = axes,
        period = year
        )

    simulation = init_single_entity(tax_benefit_system.new_scenario(), **scenario_kwargs).new_simulation()
    assert simulation.calculate_add('salaire_de_base', year) == pytest.approx([0, 0, 0, 7500, 7500, 7500, 15000, 15000, 15000])
